@@ -35,9 +35,9 @@ CPacketHELO *g_Ping = NULL;
 #endif
 std::string g_BluetoothIconPath = std::string(ICON_PATH) + std::string("/bluetooth.png");
 
-long getTicks()
+int32_t getTicks()
 {
-  long ticks;
+  int32_t ticks;
   struct timeval now;
   gettimeofday(&now, NULL);
   ticks = now.tv_sec * 1000l;
@@ -152,10 +152,11 @@ CWiiRemote::~CWiiRemote()
 /* Basicly this just sets up standard control bits */
 void CWiiRemote::SetBluetoothAddress(const char *btaddr)
 {
+  static const bdaddr_t b = {{0, 0, 0, 0, 0, 0}}; /* BDADDR_ANY */
   if (btaddr != NULL)
     str2ba(btaddr, &m_btaddr);
   else
-    bacpy(&m_btaddr, &(*BDADDR_ANY));
+    bacpy(&m_btaddr, &b);
 }
 
 void CWiiRemote::SetSensativity(float DeadX, float DeadY, int NumSamples)
@@ -183,14 +184,11 @@ void CWiiRemote::SetJoystickMap(const char *JoyMap)
     free(m_JoyMap);
   if (JoyMap != NULL)
   {
-    m_JoyMap = new char[strlen(JoyMap) + 4];
+    m_JoyMap = (char*)malloc(strlen(JoyMap) + 5);
     sprintf(m_JoyMap, "JS0:%s", JoyMap);
   }
   else
-  {
-    m_JoyMap = new char[strlen("JS0:WiiRemote")];
-    strcpy(m_JoyMap, "JS0:WiiRemote");
-  }
+    m_JoyMap = strdup("JS0:WiiRemote");
 }
 
 void CWiiRemote::Initialize(CAddress Addr, int Socket)
