@@ -1,6 +1,6 @@
 /*
-*      Copyright (C) 2005-2013 Team XBMC
-*      http://xbmc.org
+*      Copyright (C) 2005-2008 Team XBMC
+*      http://www.xbmc.org
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
@@ -13,14 +13,15 @@
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
-*  along with XBMC; see the file COPYING.  If not, see
-*  <http://www.gnu.org/licenses/>.
+*  along with XBMC; see the file COPYING.  If not, write to
+*  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+*  http://www.gnu.org/copyleft/gpl.html
 *
 */
 #include "Scraper.h"
 #include "filesystem/File.h"
 #include "filesystem/Directory.h"
-#include "filesystem/CurlFile.h"
+#include "filesystem/FileCurl.h"
 #include "AddonManager.h"
 #include "utils/ScraperParser.h"
 #include "utils/ScraperUrl.h"
@@ -33,20 +34,15 @@
 #include "FileItem.h"
 #include "utils/URIUtils.h"
 #include "utils/XMLUtils.h"
-#include "utils/StringUtils.h"
 #include "music/MusicDatabase.h"
 #include "video/VideoDatabase.h"
 #include "music/Album.h"
 #include "music/Artist.h"
-#include "Util.h"
-#include "URL.h"
-
 #include <sstream>
 
 using namespace std;
 using namespace XFILE;
 using namespace MUSIC_GRABBER;
-using namespace VIDEO;
 
 namespace ADDON
 {
@@ -132,7 +128,7 @@ CScraper::CScraper(const cp_extension_t *ext) : CAddon(ext), m_fLoaded(false)
     m_language = CAddonMgr::Get().GetExtValue(ext->configuration, "@language");
     m_requiressettings = CAddonMgr::Get().GetExtValue(ext->configuration,"@requiressettings").Equals("true");
     CStdString persistence = CAddonMgr::Get().GetExtValue(ext->configuration, "@cachepersistence");
-    if (!persistence.empty())
+    if (!persistence.IsEmpty())
       m_persistence.SetFromTimeString(persistence);
   }
   switch (Type())
@@ -158,13 +154,13 @@ CScraper::CScraper(const cp_extension_t *ext) : CAddon(ext), m_fLoaded(false)
   }
 }
 
-AddonPtr CScraper::Clone() const
+AddonPtr CScraper::Clone(const AddonPtr &self) const
 {
-  return AddonPtr(new CScraper(*this));
+  return AddonPtr(new CScraper(*this, self));
 }
 
-CScraper::CScraper(const CScraper &rhs)
-  : CAddon(rhs), m_fLoaded(false)
+CScraper::CScraper(const CScraper &rhs, const AddonPtr &self)
+  : CAddon(rhs, self), m_fLoaded(false)
 {
   m_pathContent = rhs.m_pathContent;
   m_persistence = rhs.m_persistence;
@@ -183,11 +179,11 @@ bool CScraper::SetPathSettings(CONTENT_TYPE content, const CStdString& xml)
   if (!LoadSettings())
     return false;
 
-  if (xml.empty())
+  if (xml.IsEmpty())
     return true;
 
-  CXBMCTinyXML doc;
-  doc.Parse(xml);
+  TiXmlDocument doc;
+  doc.Parse(xml.c_str());
   m_userSettingsLoaded = SettingsFromXML(doc);
 
   return m_userSettingsLoaded;
@@ -199,7 +195,7 @@ CStdString CScraper::GetPathSettings()
     return "";
 
   stringstream stream;
-  CXBMCTinyXML doc;
+  TiXmlDocument doc;
   SettingsToXML(doc);
   if (doc.RootElement())
     stream << *doc.RootElement();
@@ -235,36 +231,38 @@ void CScraper::ClearCache()
 
 // returns a vector of strings: the first is the XML output by the function; the rest
 // is XML output by chained functions, possibly recursively
-// the CCurlFile object is passed in so that URL fetches can be canceled from other threads
+// the CFileCurl object is passed in so that URL fetches can be canceled from other threads
 // throws CScraperError abort on internal failures (e.g., parse errors)
-vector<string> CScraper::Run(const CStdString& function,
+vector<CStdString> CScraper::Run(const CStdString& function,
                                  const CScraperUrl& scrURL,
-                                 CCurlFile& http,
-                                 const vector<string>* extras)
+                                 CFileCurl& http,
+                                 const vector<CStdString>* extras)
 {
   if (!Load())
     throw CScraperError();
 
   CStdString strXML = InternalRun(function,scrURL,http,extras);
-  if (strXML.empty())
+  if (strXML.IsEmpty())
   {
-    if (function != "NfoUrl" && function != "ResolveIDToUrl")
+    if (function != "NfoUrl")
       CLog::Log(LOGERROR, "%s: Unable to parse web site",__FUNCTION__);
     throw CScraperError();
   }
 
   CLog::Log(LOGDEBUG,"scraper: %s returned %s",function.c_str(),strXML.c_str());
 
-  CXBMCTinyXML doc;
-  /* all data was converted to UTF-8 before being processed by scraper */
-  doc.Parse(strXML, TIXML_ENCODING_UTF8);
+  if (!XMLUtils::HasUTF8Declaration(strXML))
+    g_charsetConverter.unknownToUTF8(strXML);
+
+  TiXmlDocument doc;
+  doc.Parse(strXML.c_str(),0,TIXML_ENCODING_UTF8);
   if (!doc.RootElement())
   {
     CLog::Log(LOGERROR, "%s: Unable to parse XML",__FUNCTION__);
     throw CScraperError();
   }
 
-  vector<string> result;
+  vector<CStdString> result;
   result.push_back(strXML);
   TiXmlElement* xchain = doc.RootElement()->FirstChildElement();
   // skip children of the root element until <url> or <chain>
@@ -277,7 +275,7 @@ vector<string> CScraper::Run(const CStdString& function,
     if (szFunction)
     {
       CScraperUrl scrURL2;
-      vector<string> extras;
+      vector<CStdString> extras;
       // for <chain>, pass the contained text as a parameter; for <url>, as URL content
       if (strcmp(xchain->Value(),"chain")==0)
       {
@@ -286,13 +284,7 @@ vector<string> CScraper::Run(const CStdString& function,
       }
       else
         scrURL2.ParseElement(xchain);
-      // Fix for empty chains. $$1 would still contain the
-      // previous value as there is no child of the xml node. 
-      // since $$1 will always either contain the data from an 
-      // url or the parameters to a chain, we can safely clear it here
-      // to fix this issue
-      m_parser.m_param[0].clear();
-      vector<string> result2 = RunNoThrow(szFunction,scrURL2,http,&extras);
+      vector<CStdString> result2 = RunNoThrow(szFunction,scrURL2,http,&extras);
       result.insert(result.end(),result2.begin(),result2.end());
     }
     xchain = xchain->NextSiblingElement();
@@ -306,12 +298,12 @@ vector<string> CScraper::Run(const CStdString& function,
 
 // just like Run, but returns an empty list instead of throwing in case of error
 // don't use in new code; errors should be handled appropriately
-vector<string> CScraper::RunNoThrow(const CStdString& function,
+std::vector<CStdString> CScraper::RunNoThrow(const CStdString& function,
   const CScraperUrl& url,
-  XFILE::CCurlFile& http,
-  const vector<string>* extras)
+  XFILE::CFileCurl& http,
+  const std::vector<CStdString>* extras)
 {
-  vector<string> vcs;
+  std::vector<CStdString> vcs;
   try
   {
     vcs = Run(function, url, http, extras);
@@ -325,8 +317,8 @@ vector<string> CScraper::RunNoThrow(const CStdString& function,
 
 CStdString CScraper::InternalRun(const CStdString& function,
                                  const CScraperUrl& scrURL,
-                                 CCurlFile& http,
-                                 const vector<string>* extras)
+                                 CFileCurl& http,
+                                 const vector<CStdString>* extras)
 {
   // walk the list of input URLs and fetch each into parser parameters
   unsigned int i;
@@ -372,7 +364,7 @@ bool CScraper::Load()
 
       if (CAddonMgr::Get().GetAddon((*itr).first, dep))
       {
-        CXBMCTinyXML doc;
+        TiXmlDocument doc;
         if (dep->Type() == ADDON_SCRAPER_LIBRARY && doc.LoadFile(dep->LibPath()))
           m_parser.AddDocument(&doc);
       }
@@ -410,29 +402,18 @@ bool CScraper::IsInUse() const
   return false;
 }
 
-bool CScraper::IsNoop()
-{
-    if (!Load())
-      throw CScraperError();
-
-    return m_parser.IsNoop();
-}
-
 // pass in contents of .nfo file; returns URL (possibly empty if none found)
 // and may populate strId, or throws CScraperError on error
 CScraperUrl CScraper::NfoUrl(const CStdString &sNfoContent)
 {
   CScraperUrl scurlRet;
 
-  if (IsNoop())
-    return scurlRet;
-
   // scraper function takes contents of .nfo file, returns XML (see below)
-  vector<string> vcsIn;
+  vector<CStdString> vcsIn;
   vcsIn.push_back(sNfoContent);
   CScraperUrl scurl;
-  CCurlFile fcurl;
-  vector<string> vcsOut = Run("NfoUrl", scurl, fcurl, &vcsIn);
+  CFileCurl fcurl;
+  vector<CStdString> vcsOut = Run("NfoUrl", scurl, fcurl, &vcsIn);
   if (vcsOut.empty() || vcsOut[0].empty())
     return scurlRet;
   if (vcsOut.size() > 1)
@@ -442,8 +423,8 @@ CScraperUrl CScraper::NfoUrl(const CStdString &sNfoContent)
   // or <url>...</url> or <url>...</url><id>...</id> on success
   for (unsigned int i=0; i < vcsOut.size(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(vcsOut[i], TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(vcsOut[i], 0, TIXML_ENCODING_UTF8);
     CheckScraperError(doc.RootElement());
 
     if (doc.RootElement())
@@ -485,68 +466,6 @@ CScraperUrl CScraper::NfoUrl(const CStdString &sNfoContent)
   return scurlRet;
 }
 
-CScraperUrl CScraper::ResolveIDToUrl(const CStdString& externalID)
-{
-  CScraperUrl scurlRet;
-  
-  // scraper function takes an external ID, returns XML (see below)
-  vector<string> vcsIn;
-  vcsIn.push_back(externalID);
-  CScraperUrl scurl;
-  CCurlFile fcurl;
-  vector<string> vcsOut = Run("ResolveIDToUrl", scurl, fcurl, &vcsIn);
-  if (vcsOut.empty() || vcsOut[0].empty())
-    return scurlRet;
-  if (vcsOut.size() > 1)
-    CLog::Log(LOGWARNING, "%s: scraper returned multiple results; using first", __FUNCTION__);
-
-  // parse returned XML: either <error> element on error, blank on failure,
-  // or <url>...</url> or <url>...</url><id>...</id> on success
-  for (unsigned int i=0; i < vcsOut.size(); ++i)
-  {
-    CXBMCTinyXML doc;
-    doc.Parse(vcsOut[i], TIXML_ENCODING_UTF8);
-    CheckScraperError(doc.RootElement());
-
-    if (doc.RootElement())
-    {
-      /*
-       NOTE: Scrapers might return invalid xml with some loose
-       elements (eg. '<url>http://some.url</url><id>123</id>').
-       Since XMLUtils::GetString() is assuming well formed xml
-       with start and end-tags we're not able to use it.
-       Check for the desired Elements instead.
-       */
-      TiXmlElement* pxeUrl=NULL;
-      TiXmlElement* pId=NULL;
-      if (!strcmp(doc.RootElement()->Value(),"details"))
-      {
-        pxeUrl = doc.RootElement()->FirstChildElement("url");
-        pId = doc.RootElement()->FirstChildElement("id");
-      }
-      else
-      {
-        pId = doc.FirstChildElement("id");
-        pxeUrl = doc.FirstChildElement("url");
-      }
-      if (pId && pId->FirstChild())
-        scurlRet.strId = pId->FirstChild()->Value();
-
-      if (pxeUrl && pxeUrl->Attribute("function"))
-        continue;
-
-      if (pxeUrl)
-        scurlRet.ParseElement(pxeUrl);
-      else if (!strcmp(doc.RootElement()->Value(), "url"))
-        scurlRet.ParseElement(doc.RootElement());
-      else
-        continue;
-      break;
-    }
-  }
-  return scurlRet;
-}
-
 static bool RelevanceSortFunction(const CScraperUrl &left, const CScraperUrl &right)
 {
   return left.relevance > right.relevance;
@@ -554,34 +473,33 @@ static bool RelevanceSortFunction(const CScraperUrl &left, const CScraperUrl &ri
 
 // fetch list of matching movies sorted by relevance (may be empty);
 // throws CScraperError on error; first called with fFirst set, then unset if first try fails
-std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStdString &sMovie,
+std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CFileCurl &fcurl, const CStdString &sMovie,
   bool fFirst)
 {
   // prepare parameters for URL creation
   CStdString sTitle, sTitleYear, sYear;
   CUtil::CleanString(sMovie, sTitle, sTitleYear, sYear, true/*fRemoveExt*/, fFirst);
 
+  if (!fFirst || Content() == CONTENT_MUSICVIDEOS)
+    sTitle.Replace("-"," ");
+
   CLog::Log(LOGDEBUG, "%s: Searching for '%s' using %s scraper "
     "(path: '%s', content: '%s', version: '%s')", __FUNCTION__, sTitle.c_str(),
     Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
-  std::vector<CScraperUrl> vcscurl;
-  if (IsNoop())
-    return vcscurl;
+  sTitle.ToLower();
 
-  if (!fFirst)
-    StringUtils::Replace(sTitle, '-',' ');
-
-  vector<string> vcsIn(1);
+  vector<CStdString> vcsIn(1);
   g_charsetConverter.utf8To(SearchStringEncoding(), sTitle, vcsIn[0]);
-  vcsIn[0] = CURL::Encode(vcsIn[0]);
-  if (!sYear.empty())
+  CURL::Encode(vcsIn[0]);
+  if (!sYear.IsEmpty())
     vcsIn.push_back(sYear);
 
   // request a search URL from the title/filename/etc.
   CScraperUrl scurl;
-  vector<string> vcsOut = Run("CreateSearchUrl", scurl, fcurl, &vcsIn);
+  vector<CStdString> vcsOut = Run("CreateSearchUrl", scurl, fcurl, &vcsIn);
+  std::vector<CScraperUrl> vcscurl;
   if (vcsOut.empty())
   {
     CLog::Log(LOGDEBUG, "%s: CreateSearchUrl failed", __FUNCTION__);
@@ -597,10 +515,10 @@ std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStd
   bool fSort(true);
   std::set<CStdString> stsDupeCheck;
   bool fResults(false);
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     if (!doc.RootElement())
     {
       CLog::Log(LOGERROR, "%s: Unable to parse XML", __FUNCTION__);
@@ -617,11 +535,7 @@ std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStd
 
     // we need to sort if returned results don't specify 'sorted="yes"'
     if (fSort)
-    {
-      const char *sorted = xhResults.Element()->Attribute("sorted");
-      if (sorted != NULL)
-        fSort = !StringUtils::EqualsNoCase(sorted, "yes");
-    }
+      fSort = CStdString(xhResults.Element()->Attribute("sorted")).CompareNoCase("yes") != 0;
 
     for (TiXmlElement *pxeMovie = xhResults.FirstChild("entity").Element();
       pxeMovie; pxeMovie = pxeMovie->NextSiblingElement())
@@ -639,9 +553,9 @@ std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStd
 
         // calculate the relavance of this hit
         CStdString sCompareTitle = scurlMovie.strTitle;
-        StringUtils::ToLower(sCompareTitle);
+        sCompareTitle.ToLower();
         CStdString sMatchTitle = sTitle;
-        StringUtils::ToLower(sMatchTitle);
+        sMatchTitle.ToLower();
 
         /*
          * Identify the best match by performing a fuzzy string compare on the search term and
@@ -660,11 +574,11 @@ std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStd
 
         // reconstruct a title for the user
         if (!sCompareYear.empty())
-          scurlMovie.strTitle += StringUtils::Format(" (%s)", sCompareYear.c_str());
+          scurlMovie.strTitle.AppendFormat(" (%s)", sCompareYear.c_str());
 
         CStdString sLanguage;
-        if (XMLUtils::GetString(pxeMovie, "language", sLanguage) && !sLanguage.empty())
-          scurlMovie.strTitle += StringUtils::Format(" (%s)", sLanguage.c_str());
+        if (XMLUtils::GetString(pxeMovie, "language", sLanguage))
+          scurlMovie.strTitle.AppendFormat(" (%s)", sLanguage.c_str());
 
         // filter for dupes from naughty scrapers
         if (stsDupeCheck.insert(scurlMovie.m_url[0].m_url + " " + scurlMovie.strTitle).second)
@@ -684,30 +598,27 @@ std::vector<CScraperUrl> CScraper::FindMovie(XFILE::CCurlFile &fcurl, const CStd
 
 // find album by artist, using fcurl for web fetches
 // returns a list of albums (empty if no match or failure)
-std::vector<CMusicAlbumInfo> CScraper::FindAlbum(CCurlFile &fcurl, const CStdString &sAlbum,
+std::vector<CMusicAlbumInfo> CScraper::FindAlbum(CFileCurl &fcurl, const CStdString &sAlbum,
   const CStdString &sArtist)
 {
   CLog::Log(LOGDEBUG, "%s: Searching for '%s - %s' using %s scraper "
     "(path: '%s', content: '%s', version: '%s')", __FUNCTION__, sArtist.c_str(),
     sAlbum.c_str(), Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
-
-  std::vector<CMusicAlbumInfo> vcali;
-  if (IsNoop())
-    return vcali;
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
   // scraper function is given the album and artist as parameters and
   // returns an XML <url> element parseable by CScraperUrl
-  std::vector<string> extras(2);
+  std::vector<CStdString> extras(2);
   g_charsetConverter.utf8To(SearchStringEncoding(), sAlbum, extras[0]);
   g_charsetConverter.utf8To(SearchStringEncoding(), sArtist, extras[1]);
-  extras[0] = CURL::Encode(extras[0]);
-  extras[1] = CURL::Encode(extras[1]);
+  CURL::Encode(extras[0]);
+  CURL::Encode(extras[1]);
   CScraperUrl scurl;
-  vector<string> vcsOut = RunNoThrow("CreateAlbumSearchUrl", scurl, fcurl, &extras);
+  vector<CStdString> vcsOut = RunNoThrow("CreateAlbumSearchUrl", scurl, fcurl, &extras);
   if (vcsOut.size() > 1)
     CLog::Log(LOGWARNING, "%s: scraper returned multiple results; using first", __FUNCTION__);
 
+  std::vector<CMusicAlbumInfo> vcali;
   if (vcsOut.empty() || vcsOut[0].empty())
     return vcali;
   scurl.ParseString(vcsOut[0]);
@@ -727,28 +638,28 @@ std::vector<CMusicAlbumInfo> CScraper::FindAlbum(CCurlFile &fcurl, const CStdStr
   vcsOut = RunNoThrow("GetAlbumSearchResults", scurl, fcurl);
 
   // parse the returned XML into a vector of album objects
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     TiXmlHandle xhDoc(&doc);
 
     for (TiXmlElement* pxeAlbum = xhDoc.FirstChild("results").FirstChild("entity").Element();
       pxeAlbum; pxeAlbum = pxeAlbum->NextSiblingElement())
     {
       CStdString sTitle;
-      if (XMLUtils::GetString(pxeAlbum, "title", sTitle) && !sTitle.empty())
+      if (XMLUtils::GetString(pxeAlbum, "title", sTitle))
       {
         CStdString sArtist;
         CStdString sAlbumName;
-        if (XMLUtils::GetString(pxeAlbum, "artist", sArtist) && !sArtist.empty())
-          sAlbumName = StringUtils::Format("%s - %s", sArtist.c_str(), sTitle.c_str());
+        if (XMLUtils::GetString(pxeAlbum, "artist", sArtist))
+          sAlbumName.Format("%s - %s", sArtist.c_str(), sTitle.c_str());
         else
           sAlbumName = sTitle;
 
         CStdString sYear;
-        if (XMLUtils::GetString(pxeAlbum, "year", sYear) && !sYear.empty())
-          sAlbumName = StringUtils::Format("%s (%s)", sAlbumName.c_str(), sYear.c_str());
+        if (XMLUtils::GetString(pxeAlbum, "year", sYear))
+          sAlbumName.Format("%s (%s)", sAlbumName.c_str(), sYear.c_str());
 
         // if no URL is provided, use the URL we got back from CreateAlbumSearchUrl
         // (e.g., in case we only got one result back and were sent to the detail page)
@@ -781,26 +692,23 @@ std::vector<CMusicAlbumInfo> CScraper::FindAlbum(CCurlFile &fcurl, const CStdStr
 
 // find artist, using fcurl for web fetches
 // returns a list of artists (empty if no match or failure)
-std::vector<CMusicArtistInfo> CScraper::FindArtist(CCurlFile &fcurl,
+std::vector<CMusicArtistInfo> CScraper::FindArtist(CFileCurl &fcurl,
   const CStdString &sArtist)
 {
   CLog::Log(LOGDEBUG, "%s: Searching for '%s' using %s scraper "
     "(file: '%s', content: '%s', version: '%s')", __FUNCTION__, sArtist.c_str(),
     Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
-
-  std::vector<CMusicArtistInfo> vcari;
-  if (IsNoop())
-    return vcari;
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
   // scraper function is given the artist as parameter and
   // returns an XML <url> element parseable by CScraperUrl
-  std::vector<string> extras(1);
+  std::vector<CStdString> extras(1);
   g_charsetConverter.utf8To(SearchStringEncoding(), sArtist, extras[0]);
-  extras[0] = CURL::Encode(extras[0]);
+  CURL::Encode(extras[0]);
   CScraperUrl scurl;
-  vector<string> vcsOut = RunNoThrow("CreateArtistSearchUrl", scurl, fcurl, &extras);
+  vector<CStdString> vcsOut = RunNoThrow("CreateArtistSearchUrl", scurl, fcurl, &extras);
 
+  std::vector<CMusicArtistInfo> vcari;
   if (vcsOut.empty() || vcsOut[0].empty())
     return vcari;
   scurl.ParseString(vcsOut[0]);
@@ -819,10 +727,10 @@ std::vector<CMusicArtistInfo> CScraper::FindArtist(CCurlFile &fcurl,
   vcsOut = RunNoThrow("GetArtistSearchResults", scurl, fcurl);
 
   // parse the returned XML into a vector of artist objects
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     if (!doc.RootElement())
     {
       CLog::Log(LOGERROR, "%s: Unable to parse XML", __FUNCTION__);
@@ -847,10 +755,7 @@ std::vector<CMusicArtistInfo> CScraper::FindArtist(CCurlFile &fcurl,
           continue;
 
         CMusicArtistInfo ari(pxnTitle->FirstChild()->Value(), scurlArtist);
-        CStdString genre;
-        XMLUtils::GetString(pxeArtist, "genre", genre);
-        if (!genre.empty())
-          ari.GetArtist().genre = StringUtils::Split(genre, g_advancedSettings.m_musicItemSeparator);
+        XMLUtils::GetString(pxeArtist, "genre", ari.GetArtist().strGenre);
         XMLUtils::GetString(pxeArtist, "year", ari.GetArtist().strBorn);
 
         vcari.push_back(ari);
@@ -861,25 +766,25 @@ std::vector<CMusicArtistInfo> CScraper::FindArtist(CCurlFile &fcurl,
 }
 
 // fetch list of episodes from URL (from video database)
-EPISODELIST CScraper::GetEpisodeList(XFILE::CCurlFile &fcurl, const CScraperUrl &scurl)
+EPISODELIST CScraper::GetEpisodeList(XFILE::CFileCurl &fcurl, const CScraperUrl &scurl)
 {
-  EPISODELIST vcep;
-  if (scurl.m_url.empty())
-    return vcep;
-  
   CLog::Log(LOGDEBUG, "%s: Searching '%s' using %s scraper "
     "(file: '%s', content: '%s', version: '%s')", __FUNCTION__,
     scurl.m_url[0].m_url.c_str(), Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
-  vector<string> vcsIn;
+  EPISODELIST vcep;
+  if (scurl.m_url.empty())
+    return vcep;
+
+  vector<CStdString> vcsIn;
   vcsIn.push_back(scurl.m_url[0].m_url);
-  vector<string> vcsOut = RunNoThrow("GetEpisodeList", scurl, fcurl, &vcsIn);
+  vector<CStdString> vcsOut = RunNoThrow("GetEpisodeList", scurl, fcurl, &vcsIn);
 
   // parse the XML response
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
+    TiXmlDocument doc;
     doc.Parse(*i);
     if (!doc.RootElement())
     {
@@ -893,15 +798,11 @@ EPISODELIST CScraper::GetEpisodeList(XFILE::CCurlFile &fcurl, const CScraperUrl
     {
       EPISODE ep;
       TiXmlElement *pxeLink = pxeMovie->FirstChildElement("url");
-      CStdString strEpNum;
-      if (pxeLink && XMLUtils::GetInt(pxeMovie, "season", ep.iSeason) &&
-        XMLUtils::GetString(pxeMovie, "epnum", strEpNum) && !strEpNum.empty())
+      if (pxeLink && XMLUtils::GetInt(pxeMovie, "season", ep.key.first) &&
+        XMLUtils::GetInt(pxeMovie, "epnum", ep.key.second))
       {
         CScraperUrl &scurlEp(ep.cScraperUrl);
-        size_t dot = strEpNum.find(".");
-        ep.iEpisode = atoi(strEpNum.c_str());
-        ep.iSubepisode = (dot != std::string::npos) ? atoi(strEpNum.substr(dot + 1).c_str()) : 0;
-        if (!XMLUtils::GetString(pxeMovie, "title", scurlEp.strTitle) || scurlEp.strTitle.empty() )
+        if (!XMLUtils::GetString(pxeMovie, "title", scurlEp.strTitle))
             scurlEp.strTitle = g_localizeStrings.Get(416);
         XMLUtils::GetString(pxeMovie, "id", scurlEp.strId);
 
@@ -922,31 +823,50 @@ EPISODELIST CScraper::GetEpisodeList(XFILE::CCurlFile &fcurl, const CScraperUrl
     }
   }
 
+  // find minimum in each season
+  map<int, int> mpMin;
+  for (EPISODELIST::const_iterator i = vcep.begin(); i != vcep.end(); ++i)
+  {
+    map<int, int>::iterator iMin = mpMin.find(i->key.first);
+    if (iMin == mpMin.end())
+      mpMin.insert(i->key);
+    else if (i->key.second < iMin->second)
+      iMin->second = i->key.second;
+  }
+
+  // correct episode numbers
+  for (EPISODELIST::iterator i = vcep.begin(); i != vcep.end(); ++i)
+  {
+    i->key.second -= mpMin[i->key.first];
+    if (mpMin[i->key.first] > 0)
+      ++i->key.second;
+  }
+
   return vcep;
 }
 
 // takes URL; returns true and populates video details on success, false otherwise
-bool CScraper::GetVideoDetails(XFILE::CCurlFile &fcurl, const CScraperUrl &scurl,
+bool CScraper::GetVideoDetails(XFILE::CFileCurl &fcurl, const CScraperUrl &scurl,
   bool fMovie/*else episode*/, CVideoInfoTag &video)
 {
   CLog::Log(LOGDEBUG, "%s: Reading %s '%s' using %s scraper "
     "(file: '%s', content: '%s', version: '%s')", __FUNCTION__,
-    fMovie ? MediaTypeMovie : MediaTypeEpisode, scurl.m_url[0].m_url.c_str(), Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
+    fMovie ? "movie" : "episode", scurl.m_url[0].m_url.c_str(), Name().c_str(), Path().c_str(),
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
   video.Reset();
   CStdString sFunc = fMovie ? "GetDetails" : "GetEpisodeDetails";
-  vector<string> vcsIn;
+  vector<CStdString> vcsIn;
   vcsIn.push_back(scurl.strId);
   vcsIn.push_back(scurl.m_url[0].m_url);
-  vector<string> vcsOut = RunNoThrow(sFunc, scurl, fcurl, &vcsIn);
+  vector<CStdString> vcsOut = RunNoThrow(sFunc, scurl, fcurl, &vcsIn);
 
   // parse XML output
   bool fRet(false);
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     if (!doc.RootElement())
     {
       CLog::Log(LOGERROR, "%s: Unable to parse XML", __FUNCTION__);
@@ -967,21 +887,21 @@ bool CScraper::GetVideoDetails(XFILE::CCurlFile &fcurl, const CScraperUrl &scurl
 }
 
 // takes a URL; returns true and populates album on success, false otherwise
-bool CScraper::GetAlbumDetails(CCurlFile &fcurl, const CScraperUrl &scurl, CAlbum &album)
+bool CScraper::GetAlbumDetails(CFileCurl &fcurl, const CScraperUrl &scurl, CAlbum &album)
 {
   CLog::Log(LOGDEBUG, "%s: Reading '%s' using %s scraper "
     "(file: '%s', content: '%s', version: '%s')", __FUNCTION__,
     scurl.m_url[0].m_url.c_str(), Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
-  vector<string> vcsOut = RunNoThrow("GetAlbumDetails", scurl, fcurl);
+  vector<CStdString> vcsOut = RunNoThrow("GetAlbumDetails", scurl, fcurl);
 
   // parse the returned XML into an album object (see CAlbum::Load for details)
   bool fRet(false);
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (CStdStringArray::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     if (!doc.RootElement())
     {
       CLog::Log(LOGERROR, "%s: Unable to parse XML", __FUNCTION__);
@@ -994,7 +914,7 @@ bool CScraper::GetAlbumDetails(CCurlFile &fcurl, const CScraperUrl &scurl, CAlbu
 
 // takes a URL (one returned from FindArtist), the original search string, and
 // returns true and populates artist on success, false on failure
-bool CScraper::GetArtistDetails(CCurlFile &fcurl, const CScraperUrl &scurl,
+bool CScraper::GetArtistDetails(CFileCurl &fcurl, const CScraperUrl &scurl,
   const CStdString &sSearch, CArtist &artist)
 {
   if (!scurl.m_url.size())
@@ -1003,21 +923,21 @@ bool CScraper::GetArtistDetails(CCurlFile &fcurl, const CScraperUrl &scurl,
   CLog::Log(LOGDEBUG, "%s: Reading '%s' ('%s') using %s scraper "
     "(file: '%s', content: '%s', version: '%s')", __FUNCTION__,
     scurl.m_url[0].m_url.c_str(), sSearch.c_str(), Name().c_str(), Path().c_str(),
-    ADDON::TranslateContent(Content()).c_str(), Version().asString().c_str());
+    ADDON::TranslateContent(Content()).c_str(), Version().c_str());
 
   // pass in the original search string for chaining to search other sites
-  vector<string> vcIn;
+  vector<CStdString> vcIn;
   vcIn.push_back(sSearch);
-  vcIn[0] = CURL::Encode(vcIn[0]);
+  CURL::Encode(vcIn[0]);
 
-  vector<string> vcsOut = RunNoThrow("GetArtistDetails", scurl, fcurl, &vcIn);
+  vector<CStdString> vcsOut = RunNoThrow("GetArtistDetails", scurl, fcurl, &vcIn);
 
   // ok, now parse the xml file
   bool fRet(false);
-  for (vector<string>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
+  for (vector<CStdString>::const_iterator i = vcsOut.begin(); i != vcsOut.end(); ++i)
   {
-    CXBMCTinyXML doc;
-    doc.Parse(*i, TIXML_ENCODING_UTF8);
+    TiXmlDocument doc;
+    doc.Parse(*i, 0, TIXML_ENCODING_UTF8);
     if (!doc.RootElement())
     {
       CLog::Log(LOGERROR, "%s: Unable to parse XML", __FUNCTION__);
