@@ -22,11 +22,12 @@
 #include "system.h"
 #ifdef HAS_PULSEAUDIO
 #include "PulseAudioDirectSound.h"
-#include "AudioContext.h"
-#include "AdvancedSettings.h"
-#include "GUISettings.h"
+#include "guilib/AudioContext.h"
+#include "settings/AdvancedSettings.h"
+#include "settings/Settings.h"
 #include "utils/log.h"
 #include "Util.h"
+#include "guilib/LocalizeStrings.h"
 
 static const char *ContextStateToString(pa_context_state s)
 {
@@ -118,6 +119,7 @@ static void StreamLatencyUpdateCallback(pa_stream *s, void *userdata)
 
 struct SinkInfoStruct
 {
+  bool passthrough;
   AudioSinkList *list;
   pa_threaded_mainloop *mainloop;
 };
@@ -125,13 +127,36 @@ struct SinkInfoStruct
 static void SinkInfo(pa_context *c, const pa_sink_info *i, int eol, void *userdata)
 {
   SinkInfoStruct *sinkStruct = (SinkInfoStruct *)userdata;
+
   if (i && i->name)
   {
-    CStdString descr = i->description;
-    CStdString sink;
-    sink.Format("pulse:%s@default", i->name);
-    sinkStruct->list->push_back(AudioSink(descr, sink));
-    CLog::Log(LOGDEBUG, "PulseAudio: Found %s with devicestring %s", descr.c_str(), sink.c_str());
+    bool       add  = false;
+    if(sinkStruct->passthrough)
+    {
+#if PA_CHECK_VERSION(1,0,0)
+      for(int idx = 0; idx < i->n_formats; ++idx)
+      {
+        if(!pa_format_info_is_pcm(i->formats[idx]))
+        {
+          add = true;
+          break;
+        }
+      }
+#endif
+    }
+    else
+      add = true;
+
+    if(add)
+    {
+      CStdString desc, sink;
+      if(sinkStruct->list->size() == 0)
+        sinkStruct->list->push_back(AudioSink(g_localizeStrings.Get(409) + " (PulseAudio)", "pulse:default@default"));
+      desc.Format("%s  (PulseAudio)", i->description);
+      sink.Format("pulse:%s@default", i->name);
+      sinkStruct->list->push_back(AudioSink(desc, sink));
+      CLog::Log(LOGDEBUG, "PulseAudio: Found %s with devicestring %s", desc.c_str(), sink.c_str());
+    }
   }
 
   pa_threaded_mainloop_signal(sinkStruct->mainloop, 0);
@@ -143,11 +168,25 @@ CPulseAudioDirectSound::CPulseAudioDirectSound()
 {
 }
 
-bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdString& device, int iChannels, unsigned int uiSamplesPerSec, unsigned int uiBitsPerSample, bool bResample, const char* strAudioCodec, bool bIsMusic, bool bPassthrough)
+bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdString& device, int iChannels, enum PCMChannels* channelMap, unsigned int uiSamplesPerSec, unsigned int uiBitsPerSample, bool bResample, bool bIsMusic, EEncoded encoded)
 {
-  CLog::Log(LOGDEBUG,"PulseAudio: Opening Channels: %i - SampleRate: %i - SampleBit: %i - Resample %s - Codec %s - IsMusic %s - IsPassthrough %s - device: %s", iChannels, uiSamplesPerSec, uiBitsPerSample, bResample ? "true" : "false", strAudioCodec, bIsMusic ? "true" : "false", bPassthrough ? "true" : "false", device.c_str());
-  if (iChannels == 0)
-    iChannels = 2;
+  m_remap.Reset();
+  m_uiDataChannels = iChannels;
+  enum PCMChannels* outLayout = NULL;
+
+  if (encoded == ENCODED_NONE && channelMap)
+  {
+    /* set the input format, and get the channel layout so we know what we need to open */
+    outLayout = m_remap.SetInputFormat(iChannels, channelMap, uiBitsPerSample / 8, uiSamplesPerSec);
+    enum PCMChannels *channel;
+    iChannels = 0;
+    for(channel = outLayout; *channel != PCM_INVALID; ++channel)
+      ++iChannels;
+
+    m_remap.SetOutputFormat(iChannels, outLayout);
+    if (m_uiDataChannels != (unsigned int)iChannels)
+      CLog::Log(LOGDEBUG, "CPulseAudioDirectSound::CPulseAudioDirectSound - Requested channels changed from %i to %i", m_uiDataChannels, iChannels);
+  }
 
   bool bAudioOnAllSpeakers(false);
   g_audioContext.SetupSpeakerConfig(iChannels, bAudioOnAllSpeakers, bIsMusic);
@@ -164,21 +203,24 @@ bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdStr
   m_uiSamplesPerSec = uiSamplesPerSec;
   m_uiBufferSize = 0;
   m_uiBitsPerSample = uiBitsPerSample;
-  m_bPassthrough = bPassthrough;
+  m_bPassthrough = encoded == ENCODED_NONE ? false : true;
   m_uiBytesPerSecond = uiSamplesPerSec * (uiBitsPerSample / 8) * iChannels;
+  m_drc = 0;
 
-  m_nCurrentVolume = g_stSettings.m_nVolumeLevel;
+  m_nCurrentVolume = g_settings.m_nVolumeLevel;
 
   m_dwPacketSize = iChannels*(uiBitsPerSample/8)*512;
   m_dwNumPackets = 16;
 
+#if !PA_CHECK_VERSION(1,0,0)
   /* Open the device */
   if (m_bPassthrough)
   {
     CLog::Log(LOGWARNING, "PulseAudio: Does not support passthrough");
     return false;
   }
- 
+#endif
+
   std::vector<CStdString> hostdevice;
   CUtil::Tokenize(device, hostdevice, "@");
 
@@ -192,27 +234,83 @@ bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdStr
 
   pa_threaded_mainloop_lock(m_MainLoop);
 
+
   struct pa_channel_map map;
 
-  m_SampleSpec.channels = iChannels;
-  m_SampleSpec.rate = uiSamplesPerSec;
-  m_SampleSpec.format = PA_SAMPLE_S16NE;  
-
-  if (!pa_sample_spec_valid(&m_SampleSpec)) 
+  // Build the channel map, we dont need to remap, but we still need PCMRemap to handle mono to dual mono stereo
+  map.channels = iChannels;
+  if (outLayout)
   {
-    CLog::Log(LOGERROR, "PulseAudio: Invalid sample spec");
-    Deinitialize();
-    return false;
+    for(int ch = 0; ch < iChannels; ++ch)
+    {
+      switch(outLayout[ch])
+      {
+        case PCM_INVALID              : break;
+        case PCM_FRONT_LEFT           : map.map[ch] = PA_CHANNEL_POSITION_FRONT_LEFT           ; break;
+        case PCM_FRONT_RIGHT          : map.map[ch] = PA_CHANNEL_POSITION_FRONT_RIGHT          ; break;
+        case PCM_FRONT_CENTER         : map.map[ch] = PA_CHANNEL_POSITION_FRONT_CENTER         ; break;
+        case PCM_BACK_CENTER          : map.map[ch] = PA_CHANNEL_POSITION_REAR_CENTER          ; break;
+        case PCM_BACK_LEFT            : map.map[ch] = PA_CHANNEL_POSITION_REAR_LEFT            ; break;
+        case PCM_BACK_RIGHT           : map.map[ch] = PA_CHANNEL_POSITION_REAR_RIGHT           ; break;
+        case PCM_LOW_FREQUENCY        : map.map[ch] = PA_CHANNEL_POSITION_LFE                  ; break;
+        case PCM_FRONT_LEFT_OF_CENTER : map.map[ch] = PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER ; break;
+        case PCM_FRONT_RIGHT_OF_CENTER: map.map[ch] = PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER; break;
+        case PCM_SIDE_LEFT            : map.map[ch] = PA_CHANNEL_POSITION_SIDE_LEFT            ; break;
+        case PCM_SIDE_RIGHT           : map.map[ch] = PA_CHANNEL_POSITION_SIDE_RIGHT           ; break;
+        case PCM_TOP_CENTER           : map.map[ch] = PA_CHANNEL_POSITION_TOP_CENTER           ; break;
+        case PCM_TOP_FRONT_LEFT       : map.map[ch] = PA_CHANNEL_POSITION_TOP_FRONT_LEFT       ; break;
+        case PCM_TOP_FRONT_RIGHT      : map.map[ch] = PA_CHANNEL_POSITION_TOP_FRONT_RIGHT      ; break;
+        case PCM_TOP_FRONT_CENTER     : map.map[ch] = PA_CHANNEL_POSITION_TOP_CENTER           ; break;
+        case PCM_TOP_BACK_LEFT        : map.map[ch] = PA_CHANNEL_POSITION_TOP_REAR_LEFT        ; break;
+        case PCM_TOP_BACK_RIGHT       : map.map[ch] = PA_CHANNEL_POSITION_TOP_REAR_RIGHT       ; break;
+        case PCM_TOP_BACK_CENTER      : map.map[ch] = PA_CHANNEL_POSITION_TOP_REAR_CENTER      ; break;
+      }
+    }
+  }
+  else
+    pa_channel_map_init_auto(&map, m_uiChannels, PA_CHANNEL_MAP_ALSA);
+
+  pa_cvolume_reset(&m_Volume, m_uiChannels);
+
+  if(m_bPassthrough)
+  {
+#if PA_CHECK_VERSION(1,0,0)
+    pa_format_info *info[1];
+    info[0] = pa_format_info_new();
+    switch(encoded)
+    {
+      case ENCODED_IEC61937_AC3 : info[0]->encoding = PA_ENCODING_AC3_IEC61937 ; break;
+      case ENCODED_IEC61937_DTS : info[0]->encoding = PA_ENCODING_DTS_IEC61937 ; break;
+      case ENCODED_IEC61937_EAC3: info[0]->encoding = PA_ENCODING_EAC3_IEC61937; break;
+      case ENCODED_IEC61937_MPEG: info[0]->encoding = PA_ENCODING_MPEG_IEC61937; break;
+      default:                    info[0]->encoding = PA_ENCODING_INVALID      ; break;
+    }
+    pa_format_info_set_rate(info[0], m_uiSamplesPerSec);
+    pa_format_info_set_channels(info[0], m_uiChannels);
+    pa_format_info_set_sample_format(info[0], PA_SAMPLE_S16NE);
+    m_Stream = pa_stream_new_extended(m_Context, "audio stream", info, 1, NULL);
+    pa_format_info_free(info[0]);
+#endif
+  }
+  else
+  {
+
+    pa_sample_spec spec;
+    spec.channels = iChannels;
+    spec.rate     = uiSamplesPerSec;
+    spec.format   = PA_SAMPLE_S16NE;
+
+    if (!pa_sample_spec_valid(&spec))
+    {
+      CLog::Log(LOGERROR, "PulseAudio: Invalid sample spec");
+      Deinitialize();
+      return false;
+    }
+
+    m_Stream = pa_stream_new(m_Context, "audio stream", &spec, &map);
   }
 
-  if (strstr(strAudioCodec, "DMO") || strstr(strAudioCodec, "FLAC") || strstr(strAudioCodec, "PCM"))
-    pa_channel_map_init_auto(&map, m_SampleSpec.channels, PA_CHANNEL_MAP_WAVEEX);
-  else
-    pa_channel_map_init_auto(&map, m_SampleSpec.channels, PA_CHANNEL_MAP_ALSA);
-
-  pa_cvolume_reset(&m_Volume, m_SampleSpec.channels);
-
-  if ((m_Stream = pa_stream_new(m_Context, "audio stream", &m_SampleSpec, &map)) == NULL)
+  if (m_Stream == NULL)
   {
     CLog::Log(LOGERROR, "PulseAudio: Could not create a stream");
     pa_threaded_mainloop_unlock(m_MainLoop);
@@ -258,7 +356,7 @@ bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdStr
     m_dwPacketSize = a->minreq;
     CLog::Log(LOGDEBUG, "PulseAudio: Default buffer attributes, maxlength=%u, tlength=%u, prebuf=%u, minreq=%u", a->maxlength, a->tlength, a->prebuf, a->minreq);
     pa_buffer_attr b;
-    b.prebuf = a->minreq * 10;
+    b.prebuf = (uint32_t)-1;
     b.minreq = a->minreq;
     b.tlength = m_uiBufferSize = a->tlength;
     b.maxlength = a->maxlength;
@@ -275,7 +373,7 @@ bool CPulseAudioDirectSound::Initialize(IAudioCallback* pCallback, const CStdStr
       CLog::Log(LOGDEBUG, "PulseAudio: Choosen buffer attributes, maxlength=%u, tlength=%u, prebuf=%u, minreq=%u", a->maxlength, a->tlength, a->prebuf, a->minreq);
     }
   }
-  
+
   pa_threaded_mainloop_unlock(m_MainLoop);
 
   m_bIsAllocated = true;
@@ -374,7 +472,7 @@ bool CPulseAudioDirectSound::Pause()
   if (!m_bIsAllocated)
     return -1;
 
-  if (m_bPause) 
+  if (m_bPause)
     return true;
 
   m_bPause = Cork(true);
@@ -417,7 +515,7 @@ long CPulseAudioDirectSound::GetCurrentVolume() const
 
 void CPulseAudioDirectSound::Mute(bool bMute)
 {
-  if (!m_bIsAllocated) 
+  if (!m_bIsAllocated)
     return;
 
   if (bMute)
@@ -434,9 +532,9 @@ bool CPulseAudioDirectSound::SetCurrentVolume(long nVolume)
   pa_threaded_mainloop_lock(m_MainLoop);
   pa_volume_t volume = pa_sw_volume_from_dB((float)nVolume*1.5f / 200.0f);
   if ( nVolume <= VOLUME_MINIMUM )
-    pa_cvolume_mute(&m_Volume, m_SampleSpec.channels);
+    pa_cvolume_mute(&m_Volume, m_uiChannels);
   else
-    pa_cvolume_set(&m_Volume, m_SampleSpec.channels, volume);
+    pa_cvolume_set(&m_Volume, m_uiChannels, volume);
   pa_operation *op = pa_context_set_sink_input_volume(m_Context, pa_stream_get_index(m_Stream), &m_Volume, NULL, NULL);
   if (op == NULL)
     CLog::Log(LOGERROR, "PulseAudio: Failed to set volume");
@@ -457,7 +555,7 @@ unsigned int CPulseAudioDirectSound::GetSpace()
   pa_threaded_mainloop_lock(m_MainLoop);
   l = pa_stream_writable_size(m_Stream);
   pa_threaded_mainloop_unlock(m_MainLoop);
-  return l;
+  return (l / m_uiChannels) * m_uiDataChannels;
 }
 
 unsigned int CPulseAudioDirectSound::AddPackets(const void* data, unsigned int len)
@@ -466,20 +564,39 @@ unsigned int CPulseAudioDirectSound::AddPackets(const void* data, unsigned int l
     return len;
 
   pa_threaded_mainloop_lock(m_MainLoop);
-  int length = std::min((int)GetSpace(), (int)len);
 
-  int rtn = pa_stream_write(m_Stream, data, length, NULL, 0, PA_SEEK_RELATIVE);
+  len = (len / m_uiDataChannels) * m_uiChannels;
+  int length = std::min((int)pa_stream_writable_size(m_Stream), (int)len);
+  int frames = length / m_uiChannels / (m_uiBitsPerSample >> 3);
+  if (frames == 0)
+  {
+    pa_threaded_mainloop_unlock(m_MainLoop);
+    return 0;
+  }
+
+  if (m_remap.CanRemap())
+  {
+    /* remap the data to the correct channels */
+    uint8_t outData[length];
+    m_remap.Remap((void *)data, outData, frames, m_drc);
+    if (pa_stream_write(m_Stream, outData, length, NULL, 0, PA_SEEK_RELATIVE) < 0)
+      CLog::Log(LOGERROR, "CPulseAudioDirectSound::AddPackets - pa_stream_write failed\n");
+
+  }
+  else
+    if (pa_stream_write(m_Stream, data, length, NULL, 0, PA_SEEK_RELATIVE) < 0)
+      CLog::Log(LOGERROR, "CPulseAudioDirectSound::AddPackets - pa_stream_write failed\n");
 
 
-  if (rtn < length && m_bRecentlyFlushed)
+  if (m_bRecentlyFlushed)
     m_bRecentlyFlushed = false;
 
   pa_threaded_mainloop_unlock(m_MainLoop);
 
-  if (m_bAutoResume)   
+  if (m_bAutoResume)
     m_bAutoResume = !Resume();
 
-  return length - rtn;
+  return (length / m_uiChannels) * m_uiDataChannels;
 }
 
 float CPulseAudioDirectSound::GetCacheTime()
@@ -487,6 +604,11 @@ float CPulseAudioDirectSound::GetCacheTime()
   return (float)(m_uiBufferSize - GetSpace()) / (float)m_uiBytesPerSecond;
 }
 
+float CPulseAudioDirectSound::GetCacheTotal()
+{
+  return (float)m_uiBufferSize / (float)m_uiBytesPerSecond;
+}
+
 float CPulseAudioDirectSound::GetDelay()
 {
   if (!m_bIsAllocated)
@@ -510,7 +632,7 @@ float CPulseAudioDirectSound::GetDelay()
 
 unsigned int CPulseAudioDirectSound::GetChunkLen()
 {
-  return m_dwPacketSize;
+  return (m_dwPacketSize / m_uiChannels) * m_uiDataChannels;
 }
 
 int CPulseAudioDirectSound::SetPlaySpeed(int iSpeed)
@@ -553,18 +675,15 @@ void CPulseAudioDirectSound::EnumerateAudioSinks(AudioSinkList& vAudioSinks, boo
     return;
   }
 
-  if (!passthrough)
-  {
     pa_threaded_mainloop_lock(mainloop);
 
     SinkInfoStruct sinkStruct;
+    sinkStruct.passthrough = passthrough;
     sinkStruct.mainloop = mainloop;
     sinkStruct.list = &vAudioSinks;
-    vAudioSinks.push_back(AudioSink("default", "pulse:default@default"));
     WaitForOperation(pa_context_get_sink_info_list(context,	SinkInfo, &sinkStruct), mainloop, "EnumerateAudioSinks");
 
     pa_threaded_mainloop_unlock(mainloop);
-  }
 
   if (mainloop)
     pa_threaded_mainloop_stop(mainloop);
@@ -615,7 +734,7 @@ bool CPulseAudioDirectSound::SetupContext(const char *host, pa_context **context
 
   /* Wait until the context is ready */
   do
-  {  
+  {
     pa_threaded_mainloop_wait(*mainloop);
     CLog::Log(LOGDEBUG, "PulseAudio: Context %s", ContextStateToString(pa_context_get_state(*context)));
   }
