@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2008 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,8 +13,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
+ *  along with XBMC; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  http://www.gnu.org/copyleft/gpl.html
  *
  */
 
@@ -25,13 +26,11 @@
 #include "GUIInfoManager.h"
 #include "utils/TimeUtils.h"
 #include "utils/log.h"
-#include "utils/SortUtils.h"
 #include "utils/StringUtils.h"
-#include "FileItem.h"
+#include "GUIStaticItem.h"
 #include "Key.h"
 #include "utils/MathUtils.h"
-#include "utils/XBMCTinyXML.h"
-#include "listproviders/IListProvider.h"
+#include "tinyXML/tinyxml.h"
 
 using namespace std;
 
@@ -41,7 +40,7 @@ using namespace std;
 #define SCROLLING_THRESHOLD 300U
 
 CGUIBaseContainer::CGUIBaseContainer(int parentID, int controlID, float posX, float posY, float width, float height, ORIENTATION orientation, const CScroller& scroller, int preloadItems)
-    : IGUIContainer(parentID, controlID, posX, posY, width, height)
+    : CGUIControl(parentID, controlID, posX, posY, width, height)
     , m_scroller(scroller)
 {
   m_cursor = 0;
@@ -51,22 +50,19 @@ CGUIBaseContainer::CGUIBaseContainer(int parentID, int controlID, float posX, fl
   m_pageControl = 0;
   m_orientation = orientation;
   m_analogScrollCount = 0;
+  m_lastItem = NULL;
+  m_staticContent = false;
+  m_staticUpdateTime = 0;
+  m_staticDefaultItem = -1;
+  m_staticDefaultAlways = false;
   m_wasReset = false;
   m_layout = NULL;
   m_focusedLayout = NULL;
   m_cacheItems = preloadItems;
-  m_scrollItemsPerFrame = 0.0f;
-  m_type = VIEW_TYPE_NONE;
-  m_listProvider = NULL;
-  m_autoScrollMoveTime = 0;
-  m_autoScrollDelayTime = 0;
-  m_autoScrollIsReversed = false;
-  m_lastRenderTime = 0;
 }
 
 CGUIBaseContainer::~CGUIBaseContainer(void)
 {
-  delete m_listProvider;
 }
 
 void CGUIBaseContainer::DoProcess(unsigned int currentTime, CDirtyRegionList &dirtyregions)
@@ -76,19 +72,10 @@ void CGUIBaseContainer::DoProcess(unsigned int currentTime, CDirtyRegionList &di
   if (m_pageChangeTimer.GetElapsedMilliseconds() > 200)
     m_pageChangeTimer.Stop();
   m_wasReset = false;
-
-  // if not visible, we reset the autoscroll timer
-  if (!IsVisible() && m_autoScrollMoveTime)
-  {
-    ResetAutoScrolling();
-  }
 }
 
 void CGUIBaseContainer::Process(unsigned int currentTime, CDirtyRegionList &dirtyregions)
 {
-  // update our auto-scrolling as necessary
-  UpdateAutoScrolling(currentTime);
-
   ValidateOffset();
 
   if (m_bInvalidated)
@@ -131,25 +118,21 @@ void CGUIBaseContainer::Process(unsigned int currentTime, CDirtyRegionList &dirt
       CGUIListItemPtr item = m_items[itemNo];
       // render our item
       if (m_orientation == VERTICAL)
-        ProcessItem(origin.x, pos, item, focused, currentTime, dirtyregions);
+        ProcessItem(origin.x, pos, item.get(), focused, currentTime, dirtyregions);
       else
-        ProcessItem(pos, origin.y, item, focused, currentTime, dirtyregions);
+        ProcessItem(pos, origin.y, item.get(), focused, currentTime, dirtyregions);
     }
     // increment our position
     pos += focused ? m_focusedLayout->Size(m_orientation) : m_layout->Size(m_orientation);
     current++;
   }
 
-  // when we are scrolling up, offset will become lower (integer division, see offset calc)
-  // to have same behaviour when scrolling down, we need to set page control to offset+1
-  UpdatePageControl(offset + (m_scroller.IsScrollingDown() ? 1 : 0));
-
-  m_lastRenderTime = currentTime;
+  UpdatePageControl(offset);
 
   CGUIControl::Process(currentTime, dirtyregions);
 }
 
-void CGUIBaseContainer::ProcessItem(float posX, float posY, CGUIListItemPtr& item, bool focused, unsigned int currentTime, CDirtyRegionList &dirtyregions)
+void CGUIBaseContainer::ProcessItem(float posX, float posY, CGUIListItem *item, bool focused, unsigned int currentTime, CDirtyRegionList &dirtyregions)
 {
   if (!m_focusedLayout || !m_layout) return;
 
@@ -179,7 +162,7 @@ void CGUIBaseContainer::ProcessItem(float posX, float posY, CGUIListItemPtr& ite
           subItem = m_lastItem->GetFocusedLayout()->GetFocusedItem();
         item->GetFocusedLayout()->SetFocusedItem(subItem ? subItem : 1);
       }
-      item->GetFocusedLayout()->Process(item.get(), m_parentID, currentTime, dirtyregions);
+      item->GetFocusedLayout()->Process(item, m_parentID, currentTime, dirtyregions);
     }
     m_lastItem = item;
   }
@@ -193,9 +176,9 @@ void CGUIBaseContainer::ProcessItem(float posX, float posY, CGUIListItemPtr& ite
       item->SetLayout(layout);
     }
     if (item->GetFocusedLayout())
-      item->GetFocusedLayout()->Process(item.get(), m_parentID, currentTime, dirtyregions);
+      item->GetFocusedLayout()->Process(item, m_parentID, currentTime, dirtyregions);
     if (item->GetLayout())
-      item->GetLayout()->Process(item.get(), m_parentID, currentTime, dirtyregions);
+      item->GetLayout()->Process(item, m_parentID, currentTime, dirtyregions);
   }
 
   g_graphicsContext.RestoreOrigin();
@@ -266,6 +249,8 @@ void CGUIBaseContainer::Render()
     g_graphicsContext.RestoreClipRegion();
   }
 
+  UpdatePageControl(offset);
+
   CGUIControl::Render();
 }
 
@@ -299,8 +284,6 @@ bool CGUIBaseContainer::OnAction(const CAction &action)
     OnJumpLetter((char)(action.GetID() & 0xff));
     return true;
   }
-  // stop the timer on any other action
-  m_matchTimer.Stop();
 
   switch (action.GetID())
   {
@@ -309,44 +292,26 @@ bool CGUIBaseContainer::OnAction(const CAction &action)
   case ACTION_MOVE_DOWN:
   case ACTION_MOVE_UP:
   case ACTION_NAV_BACK:
-  case ACTION_PREVIOUS_MENU:
     {
       if (!HasFocus()) return false;
-
       if (action.GetHoldTime() > HOLD_TIME_START &&
         ((m_orientation == VERTICAL && (action.GetID() == ACTION_MOVE_UP || action.GetID() == ACTION_MOVE_DOWN)) ||
          (m_orientation == HORIZONTAL && (action.GetID() == ACTION_MOVE_LEFT || action.GetID() == ACTION_MOVE_RIGHT))))
       { // action is held down - repeat a number of times
         float speed = std::min(1.0f, (float)(action.GetHoldTime() - HOLD_TIME_START) / (HOLD_TIME_END - HOLD_TIME_START));
-        unsigned int frameDuration = std::min(CTimeUtils::GetFrameTime() - m_lastHoldTime, 50u); // max 20fps
-
-        // maximal scroll rate is at least 30 items per second, and at most (item_rows/7) items per second
-        //  i.e. timed to take 7 seconds to traverse the list at full speed.
-        // minimal scroll rate is at least 10 items per second
-        float maxSpeed = std::max(frameDuration * 0.001f * 30, frameDuration * 0.001f * GetRows() / 7);
-        float minSpeed = frameDuration * 0.001f * 10;
-        m_scrollItemsPerFrame += std::max(minSpeed, speed*maxSpeed); // accelerate to max speed
+        unsigned int itemsPerFrame = 1;
+        if (m_lastHoldTime) // number of rows/10 items/second max speed
+          itemsPerFrame = std::max((unsigned int)1, (unsigned int)(speed * 0.0001f * GetRows() * (CTimeUtils::GetFrameTime() - m_lastHoldTime)));
         m_lastHoldTime = CTimeUtils::GetFrameTime();
-
-        if(m_scrollItemsPerFrame < 1.0f)//not enough hold time accumulated for one step
-          return true;
-
-        while (m_scrollItemsPerFrame >= 1)
-        {
-          if (action.GetID() == ACTION_MOVE_LEFT || action.GetID() == ACTION_MOVE_UP)
-            MoveUp(false);
-          else
-            MoveDown(false);
-          m_scrollItemsPerFrame--;
-        }
+        if (action.GetID() == ACTION_MOVE_LEFT || action.GetID() == ACTION_MOVE_UP)
+          while (itemsPerFrame--) MoveUp(false);
+        else
+          while (itemsPerFrame--) MoveDown(false);
         return true;
       }
       else
       {
-        //if HOLD_TIME_START is reached we need
-        //a sane initial value for calculating m_scrollItemsPerPage
-        m_lastHoldTime = CTimeUtils::GetFrameTime();
-        m_scrollItemsPerFrame = 0.0f;
+        m_lastHoldTime = 0;
         return CGUIControl::OnAction(action);
       }
     }
@@ -400,7 +365,7 @@ bool CGUIBaseContainer::OnMessage(CGUIMessage& message)
 {
   if (message.GetControlId() == GetID() )
   {
-    if (!m_listProvider)
+    if (!m_staticContent)
     {
       if (message.GetMessage() == GUI_MSG_LABEL_BIND && message.GetPointer())
       { // bind our items
@@ -466,8 +431,7 @@ bool CGUIBaseContainer::OnMessage(CGUIMessage& message)
 
 void CGUIBaseContainer::OnUp()
 {
-  CGUIAction action = GetNavigateAction(ACTION_MOVE_UP);
-  bool wrapAround = action.GetNavigation() == GetID() || !action.HasActionsMeetingCondition();
+  bool wrapAround = m_actionUp.GetNavigation() == GetID() || !m_actionUp.HasActionsMeetingCondition();
   if (m_orientation == VERTICAL && MoveUp(wrapAround))
     return;
   // with horizontal lists it doesn't make much sense to have multiselect labels
@@ -476,8 +440,7 @@ void CGUIBaseContainer::OnUp()
 
 void CGUIBaseContainer::OnDown()
 {
-  CGUIAction action = GetNavigateAction(ACTION_MOVE_DOWN);
-  bool wrapAround = action.GetNavigation() == GetID() || !action.HasActionsMeetingCondition();
+  bool wrapAround = m_actionDown.GetNavigation() == GetID() || !m_actionDown.HasActionsMeetingCondition();
   if (m_orientation == VERTICAL && MoveDown(wrapAround))
     return;
   // with horizontal lists it doesn't make much sense to have multiselect labels
@@ -486,8 +449,7 @@ void CGUIBaseContainer::OnDown()
 
 void CGUIBaseContainer::OnLeft()
 {
-  CGUIAction action = GetNavigateAction(ACTION_MOVE_LEFT);
-  bool wrapAround = action.GetNavigation() == GetID() || !action.HasActionsMeetingCondition();
+  bool wrapAround = m_actionLeft.GetNavigation() == GetID() || !m_actionLeft.HasActionsMeetingCondition();
   if (m_orientation == HORIZONTAL && MoveUp(wrapAround))
     return;
   else if (m_orientation == VERTICAL)
@@ -501,8 +463,7 @@ void CGUIBaseContainer::OnLeft()
 
 void CGUIBaseContainer::OnRight()
 {
-  CGUIAction action = GetNavigateAction(ACTION_MOVE_RIGHT);
-  bool wrapAround = action.GetNavigation() == GetID() || !action.HasActionsMeetingCondition();
+  bool wrapAround = m_actionRight.GetNavigation() == GetID() || !m_actionRight.HasActionsMeetingCondition();
   if (m_orientation == HORIZONTAL && MoveDown(wrapAround))
     return;
   else if (m_orientation == VERTICAL)
@@ -542,12 +503,12 @@ void CGUIBaseContainer::OnPrevLetter()
   }
 }
 
-void CGUIBaseContainer::OnJumpLetter(char letter, bool skip /*=false*/)
+void CGUIBaseContainer::OnJumpLetter(char letter)
 {
   if (m_matchTimer.GetElapsedMilliseconds() < letter_match_timeout)
     m_match.push_back(letter);
   else
-    m_match = StringUtils::Format("%c", letter);
+    m_match.Format("%c", letter);
 
   m_matchTimer.StartZero();
 
@@ -557,22 +518,20 @@ void CGUIBaseContainer::OnJumpLetter(char letter, bool skip /*=false*/)
 
   // find the current letter we're focused on
   unsigned int offset = CorrectOffset(GetOffset(), GetCursor());
-  unsigned int i      = (offset + ((skip) ? 1 : 0)) % m_items.size();
-  do
+  for (unsigned int i = (offset + 1) % m_items.size(); i != offset; i = (i+1) % m_items.size())
   {
     CGUIListItemPtr item = m_items[i];
-    if (0 == strnicmp(SortUtils::RemoveArticles(item->GetLabel()).c_str(), m_match.c_str(), m_match.size()))
+    if (0 == strnicmp(SSortFileItem::RemoveArticles(item->GetLabel()).c_str(), m_match.c_str(), m_match.size()))
     {
       SelectItem(i);
       return;
     }
-    i = (i+1) % m_items.size();
-  } while (i != offset);
+  }
   // no match found - repeat with a single letter
   if (m_match.size() > 1)
   {
     m_match.clear();
-    OnJumpLetter(letter, true);
+    OnJumpLetter(letter);
   }
 }
 
@@ -593,15 +552,15 @@ void CGUIBaseContainer::OnJumpSMS(int letter)
 
   // now switch to the next letter
   CStdString current = m_letterOffsets[currentLetter].second;
-  size_t startPos = (letters.find(current) + 1) % letters.size();
+  int startPos = (letters.Find(current) + 1) % letters.size();
   // now jump to letters[startPos], or another one in the same range if possible
-  size_t pos = startPos;
+  int pos = startPos;
   while (true)
   {
     // check if we can jump to this letter
-    for (size_t i = 0; i < m_letterOffsets.size(); i++)
+    for (unsigned int i = 0; i < m_letterOffsets.size(); i++)
     {
-      if (m_letterOffsets[i].second == letters.substr(pos, 1))
+      if (m_letterOffsets[i].second == letters.Mid(pos, 1))
       {
         SelectItem(m_letterOffsets[i].first);
         return;
@@ -626,7 +585,6 @@ bool CGUIBaseContainer::MoveDown(bool wrapAround)
 // scrolls the said amount
 void CGUIBaseContainer::Scroll(int amount)
 {
-  ResetAutoScrolling();
   ScrollToOffset(GetOffset() + amount);
 }
 
@@ -736,11 +694,14 @@ bool CGUIBaseContainer::OnClick(int actionID)
   int subItem = 0;
   if (actionID == ACTION_SELECT_ITEM || actionID == ACTION_MOUSE_LEFT_CLICK)
   {
-    if (m_listProvider)
+    if (m_staticContent)
     { // "select" action
       int selected = GetSelectedItem();
       if (selected >= 0 && selected < (int)m_items.size())
-        m_listProvider->OnClick(m_items[selected]);
+      {
+        CGUIStaticItemPtr item = boost::static_pointer_cast<CGUIStaticItem>(m_items[selected]);
+        item->GetClickActions().Execute(GetID(), GetParentID());
+      }
       return true;
     }
     // grab the currently focused subitem (if applicable)
@@ -761,7 +722,7 @@ CStdString CGUIBaseContainer::GetDescription() const
   {
     CGUIListItemPtr pItem = m_items[item];
     if (pItem->m_bIsFolder)
-      strLabel = StringUtils::Format("[%s]", pItem->GetLabel().c_str());
+      strLabel.Format("[%s]", pItem->GetLabel().c_str());
     else
       strLabel = pItem->GetLabel();
   }
@@ -773,14 +734,14 @@ void CGUIBaseContainer::SetFocus(bool bOnOff)
   if (bOnOff != HasFocus())
   {
     SetInvalid();
-    m_lastItem.reset();
+    m_lastItem = NULL;
   }
   CGUIControl::SetFocus(bOnOff);
 }
 
 void CGUIBaseContainer::SaveStates(vector<CControlState> &states)
 {
-  if (!m_listProvider || !m_listProvider->AlwaysFocusDefaultItem())
+  if (!m_staticDefaultAlways)
     states.push_back(CControlState(GetID(), GetSelectedItem()));
 }
 
@@ -802,24 +763,17 @@ void CGUIBaseContainer::ValidateOffset()
 
 void CGUIBaseContainer::AllocResources()
 {
-  CGUIControl::AllocResources();
   CalculateLayout();
-  if (m_listProvider)
-  {
-    UpdateListProvider(true);
-    SelectItem(m_listProvider->GetDefaultItem());
-  }
+  if (m_staticDefaultItem != -1) // select default item
+    SelectStaticItemById(m_staticDefaultItem);
 }
 
 void CGUIBaseContainer::FreeResources(bool immediately)
 {
   CGUIControl::FreeResources(immediately);
-  if (m_listProvider)
-  {
-    if (immediately)
-      Reset();
-
-    m_listProvider->Reset(immediately);
+  if (m_staticContent)
+  { // free any static content
+    Reset();
   }
   m_scroller.Stop();
 }
@@ -828,7 +782,7 @@ void CGUIBaseContainer::UpdateLayout(bool updateAllItems)
 {
   if (updateAllItems)
   { // free memory of items
-    for (iItems it = m_items.begin(); it != m_items.end(); ++it)
+    for (iItems it = m_items.begin(); it != m_items.end(); it++)
       (*it)->FreeMemory();
   }
   // and recalculate the layout
@@ -867,46 +821,44 @@ void CGUIBaseContainer::UpdateVisibility(const CGUIListItem *item)
       (m_focusedLayout && !m_focusedLayout->CheckCondition()))
   {
     // and do it
-    int itemIndex = GetSelectedItem();
+    int item = GetSelectedItem();
     UpdateLayout(true); // true to refresh all items
-    SelectItem(itemIndex);
+    SelectItem(item);
   }
 
-  UpdateListProvider();
-}
-
-void CGUIBaseContainer::UpdateListProvider(bool forceRefresh /* = false */)
-{
-  if (m_listProvider)
-  {
-    if (m_listProvider->Update(forceRefresh))
+  if (m_staticContent)
+  { // update our item list with our new content, but only add those items that should
+    // be visible.  Save the previous item and keep it if we are adding that one.
+    CGUIListItem *lastItem = m_lastItem;
+    int selected = GetSelectedItem();
+    CGUIListItem* selectedItem = (selected >= 0 && (unsigned int)selected < m_items.size()) ? m_items[selected].get() : NULL;
+    Reset();
+    bool updateItems = false;
+    if (!m_staticUpdateTime)
+      m_staticUpdateTime = CTimeUtils::GetFrameTime();
+    if (CTimeUtils::GetFrameTime() - m_staticUpdateTime > 1000)
     {
-      // save the current item
-      int currentItem = GetSelectedItem();
-      CGUIListItem *current = (currentItem >= 0 && currentItem < (int)m_items.size()) ? m_items[currentItem].get() : NULL;
-      Reset();
-      m_listProvider->Fetch(m_items);
-      SetPageControlRange();
-      // update the newly selected item
-      bool found = false;
-      for (int i = 0; i < (int)m_items.size(); i++)
+      m_staticUpdateTime = CTimeUtils::GetFrameTime();
+      updateItems = true;
+    }
+    for (unsigned int i = 0; i < m_staticItems.size(); ++i)
+    {
+      CGUIStaticItemPtr item = boost::static_pointer_cast<CGUIStaticItem>(m_staticItems[i]);
+      if (item->UpdateVisibility(GetParentID()))
+        MarkDirtyRegion();
+      if (item->IsVisible())
       {
-        if (m_items[i].get() == current)
-        {
-          found = true;
-          if (i != currentItem)
-          {
-            SelectItem(i);
-            break;
-          }
-        }
+        m_items.push_back(item);
+        if (item.get() == lastItem)
+          m_lastItem = lastItem;
+        // if item is selected and it changed position, re-select it
+        if (item.get() == selectedItem && selected != (int)m_items.size() - 1)
+          SelectItem(m_items.size() - 1);
       }
-      if (!found && currentItem >= (int)m_items.size())
-        SelectItem(m_items.size()-1);
-      SetInvalid();
+      // update any properties
+      if (updateItems)
+        item->UpdateProperties(GetParentID());
     }
-    // always update the scroll by letter, as the list provider may have altered labels
-    // while not actually changing the list items.
     UpdateScrollByLetter();
   }
 }
@@ -942,9 +894,7 @@ void CGUIBaseContainer::UpdateScrollByLetter()
     // The letter offset jumping is only for ASCII characters at present, and
     // our checks are all done in uppercase
     CStdString nextLetter;
-    std::wstring character = item->GetSortLabel().substr(0, 1);
-    StringUtils::ToUpper(character);
-    g_charsetConverter.wToUTF8(character, nextLetter);
+    g_charsetConverter.wToUTF8(item->GetSortLabel().Left(1).ToUpper(), nextLetter);
     if (currentMatch != nextLetter)
     {
       currentMatch = nextLetter;
@@ -998,42 +948,6 @@ void CGUIBaseContainer::ScrollToOffset(int offset)
   SetOffset(offset);
 }
 
-void CGUIBaseContainer::SetAutoScrolling(const TiXmlNode *node)
-{
-  if (!node) return;
-  const TiXmlElement *scroll = node->FirstChildElement("autoscroll");
-  if (scroll)
-  {
-    scroll->Attribute("time", &m_autoScrollMoveTime);
-    if (scroll->Attribute("reverse"))
-      m_autoScrollIsReversed = true;
-    if (scroll->FirstChild())
-      m_autoScrollCondition = g_infoManager.Register(scroll->FirstChild()->ValueStr(), GetParentID());
-  }
-}
-
-void CGUIBaseContainer::ResetAutoScrolling()
-{
-  m_autoScrollDelayTime = 0;
-}
-
-void CGUIBaseContainer::UpdateAutoScrolling(unsigned int currentTime)
-{
-  if (m_autoScrollCondition && m_autoScrollCondition->Get())
-  {
-    if (m_lastRenderTime)
-      m_autoScrollDelayTime += currentTime - m_lastRenderTime;
-    if (m_autoScrollDelayTime > (unsigned int)m_autoScrollMoveTime && !m_scroller.IsScrolling())
-    { // delay is finished - start moving
-      m_autoScrollDelayTime = 0;
-      // Move up or down whether reversed moving is true or false
-      m_autoScrollIsReversed ? MoveUp(true) : MoveDown(true);
-    }
-  }
-  else
-    ResetAutoScrolling();
-}
-
 void CGUIBaseContainer::SetContainerMoving(int direction)
 {
   if (direction)
@@ -1060,8 +974,7 @@ void CGUIBaseContainer::Reset()
 {
   m_wasReset = true;
   m_items.clear();
-  m_lastItem.reset();
-  ResetAutoScrolling();
+  m_lastItem = NULL;
 }
 
 void CGUIBaseContainer::LoadLayout(TiXmlElement *layout)
@@ -1084,19 +997,33 @@ void CGUIBaseContainer::LoadLayout(TiXmlElement *layout)
   }
 }
 
-void CGUIBaseContainer::LoadListProvider(TiXmlElement *content, int defaultItem, bool defaultAlways)
+void CGUIBaseContainer::LoadContent(TiXmlElement *content)
 {
-  delete m_listProvider;
-  m_listProvider = IListProvider::Create(content, GetParentID());
-  if (m_listProvider)
-    m_listProvider->SetDefaultItem(defaultItem, defaultAlways);
+  TiXmlElement *root = content->FirstChildElement("content");
+  if (!root)
+    return;
+
+  vector<CGUIListItemPtr> items;
+  TiXmlElement *item = root->FirstChildElement("item");
+  while (item)
+  {
+    if (item->FirstChild())
+    {
+      CGUIStaticItemPtr newItem(new CGUIStaticItem(item, GetParentID()));
+      items.push_back(newItem);
+    }
+    item = item->NextSiblingElement("item");
+  }
+  SetStaticContent(items);
 }
 
-void CGUIBaseContainer::SetListProvider(IListProvider *provider)
+void CGUIBaseContainer::SetStaticContent(const vector<CGUIListItemPtr> &items)
 {
-  delete m_listProvider;
-  m_listProvider = provider;
-  UpdateListProvider(true);
+  m_staticContent = true;
+  m_staticUpdateTime = 0;
+  m_staticItems.clear();
+  m_staticItems.assign(items.begin(), items.end());
+  UpdateVisibility();
 }
 
 void CGUIBaseContainer::SetRenderOffset(const CPoint &offset)
@@ -1104,6 +1031,12 @@ void CGUIBaseContainer::SetRenderOffset(const CPoint &offset)
   m_renderOffset = offset;
 }
 
+void CGUIBaseContainer::SetType(VIEW_TYPE type, const CStdString &label)
+{
+  m_type = type;
+  m_label = label;
+}
+
 void CGUIBaseContainer::FreeMemory(int keepStart, int keepEnd)
 {
   if (keepStart < keepEnd)
@@ -1163,8 +1096,6 @@ bool CGUIBaseContainer::GetCondition(int condition, int data) const
     }
   case CONTAINER_SCROLLING:
     return (m_scrollTimer.GetElapsedMilliseconds() > std::max(m_scroller.GetDuration(), SCROLLING_THRESHOLD) || m_pageChangeTimer.IsRunning());
-  case CONTAINER_ISUPDATING:
-    return (m_listProvider) ? m_listProvider->IsUpdating() : false;
   default:
     return false;
   }
@@ -1213,21 +1144,21 @@ CStdString CGUIBaseContainer::GetLabel(int info) const
   switch (info)
   {
   case CONTAINER_NUM_PAGES:
-    label = StringUtils::Format("%u", (GetRows() + m_itemsPerPage - 1) / m_itemsPerPage);
+    label.Format("%u", (GetRows() + m_itemsPerPage - 1) / m_itemsPerPage);
     break;
   case CONTAINER_CURRENT_PAGE:
-    label = StringUtils::Format("%u", GetCurrentPage());
+    label.Format("%u", GetCurrentPage());
     break;
   case CONTAINER_POSITION:
-    label = StringUtils::Format("%i", GetCursor());
+    label.Format("%i", GetCursor());
     break;
   case CONTAINER_NUM_ITEMS:
     {
       unsigned int numItems = GetNumItems();
       if (numItems && m_items[0]->IsFileItem() && (boost::static_pointer_cast<CFileItem>(m_items[0]))->IsParentFolder())
-        label = StringUtils::Format("%u", numItems-1);
+        label.Format("%u", numItems-1);
       else
-        label = StringUtils::Format("%u", numItems);
+        label.Format("%u", numItems);
     }
     break;
   default:
@@ -1243,7 +1174,7 @@ int CGUIBaseContainer::GetCurrentPage() const
   return GetOffset() / m_itemsPerPage + 1;
 }
 
-void CGUIBaseContainer::GetCacheOffsets(int &cacheBefore, int &cacheAfter) const
+void CGUIBaseContainer::GetCacheOffsets(int &cacheBefore, int &cacheAfter)
 {
   if (m_scroller.IsScrollingDown())
   {
@@ -1276,21 +1207,29 @@ void CGUIBaseContainer::SetOffset(int offset)
 
 bool CGUIBaseContainer::CanFocus() const
 {
-  if (CGUIControl::CanFocus())
+  return (!m_items.empty() && CGUIControl::CanFocus());
+}
+
+void CGUIBaseContainer::SelectStaticItemById(int id)
+{
+  if (m_staticContent)
   {
-    /*
-     We allow focus if we have items available or if we have a list provider
-     that's in the process of updating.
-     */
-    return !m_items.empty() || (m_listProvider && m_listProvider->IsUpdating());
+    for (unsigned int i = 0 ; i < m_items.size() ; i++)
+    {
+      CGUIStaticItemPtr item = boost::static_pointer_cast<CGUIStaticItem>(m_items[i]);
+      if (item->m_iprogramCount == id)
+      {
+        SelectItem(i);
+        return;
+      }
+    }
   }
-  return false;
 }
 
 void CGUIBaseContainer::OnFocus()
 {
-  if (m_listProvider && m_listProvider->AlwaysFocusDefaultItem())
-    SelectItem(m_listProvider->GetDefaultItem());
+  if (m_staticDefaultAlways)
+    SelectStaticItemById(m_staticDefaultItem);
 
   CGUIControl::OnFocus();
 }
