@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2008 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,8 +13,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
+ *  along with XBMC; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  http://www.gnu.org/copyleft/gpl.html
  *
  */
 
@@ -26,6 +27,10 @@
 #include "GUIControlFactory.h"
 #include "GUIControlGroup.h"
 #include "GUIControlProfiler.h"
+#include "settings/Settings.h"
+#ifdef PRE_SKIN_VERSION_9_10_COMPATIBILITY
+#include "GUIEditControl.h"
+#endif
 
 #include "addons/Skin.h"
 #include "GUIInfoManager.h"
@@ -36,9 +41,7 @@
 #include "utils/XMLUtils.h"
 #include "GUIAudioManager.h"
 #include "Application.h"
-#include "ApplicationMessenger.h"
 #include "utils/Variant.h"
-#include "utils/StringUtils.h"
 
 #ifdef HAS_PERFORMANCE_SAMPLE
 #include "utils/PerformanceSample.h"
@@ -46,21 +49,17 @@
 
 using namespace std;
 
-bool CGUIWindow::icompare::operator()(const CStdString &s1, const CStdString &s2) const
-{
-  return StringUtils::CompareNoCase(s1, s2) < 0;
-}
-
 CGUIWindow::CGUIWindow(int id, const CStdString &xmlFile)
 {
   SetID(id);
   SetProperty("xmlfile", xmlFile);
+  m_idRange = 1;
   m_lastControlID = 0;
   m_overlayState = OVERLAY_STATE_PARENT_WINDOW;   // Use parent or previous window's state
   m_isDialog = false;
   m_needsScaling = true;
   m_windowLoaded = false;
-  m_loadType = LOAD_EVERY_TIME;
+  m_loadOnDemand = true;
   m_closing = false;
   m_active = false;
   m_renderOrder = 0;
@@ -70,13 +69,10 @@ CGUIWindow::CGUIWindow(int id, const CStdString &xmlFile)
   m_manualRunActions = false;
   m_exclusiveMouseControl = 0;
   m_clearBackground = 0xff000000; // opaque black -> always clear
-  m_windowXMLRootElement = NULL;
 }
 
 CGUIWindow::~CGUIWindow(void)
-{
-  delete m_windowXMLRootElement;
-}
+{}
 
 bool CGUIWindow::Load(const CStdString& strFileName, bool bContainsPath)
 {
@@ -91,21 +87,7 @@ bool CGUIWindow::Load(const CStdString& strFileName, bool bContainsPath)
   int64_t start;
   start = CurrentHostCounter();
 #endif
-  const char* strLoadType;
-  switch (m_loadType)
-  {
-  case LOAD_ON_GUI_INIT:
-    strLoadType = "LOAD_ON_GUI_INIT";
-    break;
-  case KEEP_IN_MEMORY:
-    strLoadType = "KEEP_IN_MEMORY";
-    break;
-  case LOAD_EVERY_TIME:
-  default:
-    strLoadType = "LOAD_EVERY_TIME";
-    break;
-  }
-  CLog::Log(LOGINFO, "Loading skin file: %s, load type: %s", strFileName.c_str(), strLoadType);
+  CLog::Log(LOGINFO, "Loading skin file: %s", strFileName.c_str());
   
   // Find appropriate skin folder + resolution to load from
   CStdString strPath;
@@ -115,9 +97,7 @@ bool CGUIWindow::Load(const CStdString& strFileName, bool bContainsPath)
   else
   {
     // FIXME: strLowerPath needs to eventually go since resToUse can get incorrectly overridden
-    std::string strFileNameLower = strFileName;
-    StringUtils::ToLower(strFileNameLower);
-    strLowerPath =  g_SkinInfo->GetSkinPath(strFileNameLower, &m_coordsRes);
+    strLowerPath =  g_SkinInfo->GetSkinPath(CStdString(strFileName).ToLower(), &m_coordsRes);
     strPath = g_SkinInfo->GetSkinPath(strFileName, &m_coordsRes);
   }
 
@@ -134,47 +114,32 @@ bool CGUIWindow::Load(const CStdString& strFileName, bool bContainsPath)
 
 bool CGUIWindow::LoadXML(const CStdString &strPath, const CStdString &strLowerPath)
 {
-  // load window xml if we don't have it stored yet
-  if (!m_windowXMLRootElement)
+  TiXmlDocument xmlDoc;
+  if ( !xmlDoc.LoadFile(strPath) && !xmlDoc.LoadFile(CStdString(strPath).ToLower()) && !xmlDoc.LoadFile(strLowerPath))
   {
-    CXBMCTinyXML xmlDoc;
-    std::string strPathLower = strPath;
-    StringUtils::ToLower(strPathLower);
-    if (!xmlDoc.LoadFile(strPath) && !xmlDoc.LoadFile(strPathLower) && !xmlDoc.LoadFile(strLowerPath))
-    {
-      CLog::Log(LOGERROR, "unable to load:%s, Line %d\n%s", strPath.c_str(), xmlDoc.ErrorRow(), xmlDoc.ErrorDesc());
-      SetID(WINDOW_INVALID);
-      return false;
-    }
-    m_windowXMLRootElement = (TiXmlElement*)xmlDoc.RootElement()->Clone();
+    CLog::Log(LOGERROR, "unable to load:%s, Line %d\n%s", strPath.c_str(), xmlDoc.ErrorRow(), xmlDoc.ErrorDesc());
+    SetID(WINDOW_INVALID);
+    return false;
   }
-  else
-    CLog::Log(LOGDEBUG, "Using already stored xml root node for %s", strPath.c_str());
 
-  return Load(m_windowXMLRootElement);
+  return Load(xmlDoc);
 }
 
-bool CGUIWindow::Load(TiXmlElement* pRootElement)
+bool CGUIWindow::Load(TiXmlDocument &xmlDoc)
 {
-  if (!pRootElement)
-    return false;
-  
+  TiXmlElement* pRootElement = xmlDoc.RootElement();
   if (strcmpi(pRootElement->Value(), "window"))
   {
     CLog::Log(LOGERROR, "file : XML file doesnt contain <window>");
     return false;
   }
 
-  // we must create copy of root element as we will manipulate it when resolving includes
-  // and we don't want original root element to change
-  pRootElement = (TiXmlElement*)pRootElement->Clone();
-
   // set the scaling resolution so that any control creation or initialisation can
   // be done with respect to the correct aspect ratio
   g_graphicsContext.SetScalingResolution(m_coordsRes, m_needsScaling);
 
-  // Resolve any includes that may be present and save conditions used to do it
-  g_SkinInfo->ResolveIncludes(pRootElement, &m_xmlIncludeConditions);
+  // Resolve any includes that may be present
+  g_SkinInfo->ResolveIncludes(pRootElement);
   // now load in the skin file
   SetDefaults();
 
@@ -226,8 +191,6 @@ bool CGUIWindow::Load(TiXmlElement* pRootElement)
     {
       XMLUtils::GetFloat(pChild, "posx", m_posX);
       XMLUtils::GetFloat(pChild, "posy", m_posY);
-      XMLUtils::GetFloat(pChild, "left", m_posX);
-      XMLUtils::GetFloat(pChild, "top", m_posY);
 
       TiXmlElement *originElement = pChild->FirstChildElement("origin");
       while (originElement)
@@ -254,7 +217,7 @@ bool CGUIWindow::Load(TiXmlElement* pRootElement)
       {
         if (strcmpi(pControl->Value(), "control") == 0)
         {
-          LoadControl(pControl, NULL, CRect(0, 0, (float)m_coordsRes.iWidth, (float)m_coordsRes.iHeight));
+          LoadControl(pControl, NULL);
         }
         pControl = pControl->NextSiblingElement();
       }
@@ -272,15 +235,22 @@ bool CGUIWindow::Load(TiXmlElement* pRootElement)
 
   m_windowLoaded = true;
   OnWindowLoaded();
-  delete pRootElement;
   return true;
 }
 
-void CGUIWindow::LoadControl(TiXmlElement* pControl, CGUIControlGroup *pGroup, const CRect &rect)
+void CGUIWindow::LoadControl(TiXmlElement* pControl, CGUIControlGroup *pGroup)
 {
   // get control type
   CGUIControlFactory factory;
 
+  CRect rect(0, 0, (float)m_coordsRes.iWidth, (float)m_coordsRes.iHeight);
+  if (pGroup)
+  {
+    rect.x1 = pGroup->GetXPosition();
+    rect.y1 = pGroup->GetYPosition();
+    rect.x2 = rect.x1 + pGroup->GetWidth();
+    rect.y2 = rect.y1 + pGroup->GetHeight();
+  }
   CGUIControl* pGUIControl = factory.Create(GetID(), rect, pControl);
   if (pGUIControl)
   {
@@ -303,13 +273,10 @@ void CGUIWindow::LoadControl(TiXmlElement* pControl, CGUIControlGroup *pGroup, c
     // if the new control is a group, then add it's controls
     if (pGUIControl->IsGroup())
     {
-      CGUIControlGroup *grp = (CGUIControlGroup *)pGUIControl;
       TiXmlElement *pSubControl = pControl->FirstChildElement("control");
-      CRect grpRect(grp->GetXPosition(), grp->GetYPosition(),
-                    grp->GetXPosition() + grp->GetWidth(), grp->GetYPosition() + grp->GetHeight());
       while (pSubControl)
       {
-        LoadControl(pSubControl, grp, grpRect);
+        LoadControl(pSubControl, (CGUIControlGroup *)pGUIControl);
         pSubControl = pSubControl->NextSiblingElement("control");
       }
     }
@@ -330,15 +297,10 @@ void CGUIWindow::CenterWindow()
 void CGUIWindow::DoProcess(unsigned int currentTime, CDirtyRegionList &dirtyregions)
 {
   g_graphicsContext.SetRenderingResolution(m_coordsRes, m_needsScaling);
-  g_graphicsContext.AddGUITransform();
+  unsigned int size = g_graphicsContext.AddGUITransform();
   CGUIControlGroup::DoProcess(currentTime, dirtyregions);
-  g_graphicsContext.RemoveTransform();
-
-  // check if currently focused control can have it
-  // and fallback to default control if not
-  CGUIControl* focusedControl = GetFocusedControl();
-  if (focusedControl && !focusedControl->CanFocus())
-    SET_CONTROL_FOCUS(m_defaultControl, 0);
+  if (size != g_graphicsContext.RemoveTransform())
+    CLog::Log(LOGERROR, "Unbalanced UI transforms (was %d)", size);
 }
 
 void CGUIWindow::DoRender()
@@ -351,22 +313,23 @@ void CGUIWindow::DoRender()
 
   g_graphicsContext.SetRenderingResolution(m_coordsRes, m_needsScaling);
 
-  g_graphicsContext.AddGUITransform();
+  unsigned int size = g_graphicsContext.AddGUITransform();
   CGUIControlGroup::DoRender();
-  g_graphicsContext.RemoveTransform();
+  if (size != g_graphicsContext.RemoveTransform())
+    CLog::Log(LOGERROR, "Unbalanced UI transforms (was %d)", size);
 
   if (CGUIControlProfiler::IsRunning()) CGUIControlProfiler::Instance().EndFrame();
 }
 
-void CGUIWindow::AfterRender()
+void CGUIWindow::Render()
 {
+  CGUIControlGroup::Render();
   // Check to see if we should close at this point
   // We check after the controls have finished rendering, as we may have to close due to
   // the controls rendering after the window has finished it's animation
   // we call the base class instead of this class so that we can find the change
   if (m_closing && !CGUIControlGroup::IsAnimating(ANIM_TYPE_WINDOW_CLOSE))
     Close(true);
-
 }
 
 void CGUIWindow::Close_Internal(bool forceClose /*= false*/, int nextWindowID /*= 0*/, bool enableSound /*= true*/)
@@ -392,17 +355,17 @@ void CGUIWindow::Close_Internal(bool forceClose /*= false*/, int nextWindowID /*
   }
 
   m_closing = false;
-  CGUIMessage msg(GUI_MSG_WINDOW_DEINIT, 0, 0, nextWindowID);
+  CGUIMessage msg(GUI_MSG_WINDOW_DEINIT, 0, 0);
   OnMessage(msg);
 }
 
-void CGUIWindow::Close(bool forceClose /*= false*/, int nextWindowID /*= 0*/, bool enableSound /*= true*/, bool bWait /* = true */)
+void CGUIWindow::Close(bool forceClose /*= false*/, int nextWindowID /*= 0*/, bool enableSound /*= true*/)
 {
   if (!g_application.IsCurrentThread())
   {
     // make sure graphics lock is not held
     CSingleExit leaveIt(g_graphicsContext);
-    CApplicationMessenger::Get().Close(this, forceClose, bWait, nextWindowID, enableSound);
+    g_application.getApplicationMessenger().Close(this, forceClose, true, nextWindowID, enableSound);
   }
   else
     Close_Internal(forceClose, nextWindowID, enableSound);
@@ -439,7 +402,7 @@ CPoint CGUIWindow::GetPosition() const
   for (unsigned int i = 0; i < m_origins.size(); i++)
   {
     // no condition implies true
-    if (!m_origins[i].condition || m_origins[i].condition->Get())
+    if (!m_origins[i].condition || g_infoManager.GetBoolValue(m_origins[i].condition))
     { // found origin
       return CPoint(m_origins[i].x, m_origins[i].y);
     }
@@ -458,7 +421,7 @@ EVENT_RESULT CGUIWindow::OnMouseAction(const CAction &action)
   CMouseEvent event(action.GetID(), action.GetHoldTime(), action.GetAmount(2), action.GetAmount(3));
   if (m_exclusiveMouseControl)
   {
-    CGUIControl *child = GetControl(m_exclusiveMouseControl);
+    CGUIControl *child = (CGUIControl *)GetControl(m_exclusiveMouseControl);
     if (child)
     {
       CPoint renderPos = child->GetRenderPosition() - CPoint(child->GetXPosition(), child->GetYPosition());
@@ -495,7 +458,7 @@ void CGUIWindow::OnInitWindow()
     g_audioManager.PlayWindowSound(GetID(), SOUND_INIT);
 
   // set our rendered state
-  m_hasProcessed = false;
+  m_hasRendered = false;
   m_closing = false;
   m_active = true;
   ResetAnimations();  // we need to reset our animations as those windows that don't dynamically allocate
@@ -536,13 +499,6 @@ bool CGUIWindow::OnMessage(CGUIMessage& message)
 {
   switch ( message.GetMessage() )
   {
-  case GUI_MSG_WINDOW_LOAD:
-    {
-      Initialize();
-      return true;
-    }
-    break;
-      
   case GUI_MSG_WINDOW_INIT:
     {
       CLog::Log(LOGDEBUG, "------ Window Init (%s) ------", GetProperty("xmlfile").c_str());
@@ -575,20 +531,6 @@ bool CGUIWindow::OnMessage(CGUIMessage& message)
       }
       break;
     }
-  
-  case GUI_MSG_UNFOCUS_ALL:
-    {
-      //unfocus the current focused control in this window
-      CGUIControl *control = GetFocusedControl();
-      if(control)
-      {
-        //tell focused control that it has lost the focus
-        CGUIMessage msgLostFocus(GUI_MSG_LOSTFOCUS, GetID(), control->GetID(), control->GetID());
-        control->OnMessage(msgLostFocus);
-        CLog::Log(LOGDEBUG, "Unfocus WindowID: %i, ControlID: %i",GetID(), control->GetID());
-      }
-      return true;
-    }
 
   case GUI_MSG_SELCHANGED:
     {
@@ -638,7 +580,7 @@ bool CGUIWindow::OnMessage(CGUIMessage& message)
 
         // get the control to focus
         CGUIControl* pFocusedControl = GetFirstFocusableControl(message.GetControlId());
-        if (!pFocusedControl) pFocusedControl = GetControl(message.GetControlId());
+        if (!pFocusedControl) pFocusedControl = (CGUIControl *)GetControl(message.GetControlId());
 
         // and focus it
         if (pFocusedControl)
@@ -706,11 +648,6 @@ bool CGUIWindow::OnMessage(CGUIMessage& message)
   return SendControlMessage(message);
 }
 
-bool CGUIWindow::NeedXMLReload()
-{
-  return !m_windowLoaded || g_infoManager.ConditionsChangedValues(m_xmlIncludeConditions);
-}
-
 void CGUIWindow::AllocResources(bool forceLoad /*= FALSE */)
 {
   CSingleLock lock(g_graphicsContext);
@@ -719,22 +656,13 @@ void CGUIWindow::AllocResources(bool forceLoad /*= FALSE */)
   int64_t start;
   start = CurrentHostCounter();
 #endif
-  // use forceLoad to determine if xml file needs loading
-  forceLoad |= NeedXMLReload() || (m_loadType == LOAD_EVERY_TIME);
-
-  // if window is loaded and load is forced we have to free window resources first
-  if (m_windowLoaded && forceLoad)
-    FreeResources(true);
-
-  if (forceLoad)
-  {
-    CStdString xmlFile = GetProperty("xmlfile").asString();
-    if (xmlFile.size())
-    {
-      bool bHasPath = xmlFile.find("\\") != std::string::npos || xmlFile.find("/") != std::string::npos;
-      Load(xmlFile,bHasPath);
-    }
-  }
+  // load skin xml fil
+  CStdString xmlFile = GetProperty("xmlfile").asString();
+  bool bHasPath=false;
+  if (xmlFile.Find("\\") > -1 || xmlFile.Find("/") > -1 )
+    bHasPath = true;
+  if (xmlFile.size() && (forceLoad || m_loadOnDemand || !m_windowLoaded))
+    Load(xmlFile,bHasPath);
 
   int64_t slend;
   slend = CurrentHostCounter();
@@ -746,13 +674,7 @@ void CGUIWindow::AllocResources(bool forceLoad /*= FALSE */)
   int64_t end, freq;
   end = CurrentHostCounter();
   freq = CurrentHostFrequency();
-  if (forceLoad)
-    CLog::Log(LOGDEBUG,"Alloc resources: %.2fms  (%.2f ms skin load)", 1000.f * (end - start) / freq, 1000.f * (slend - start) / freq);
-  else
-  {
-    CLog::Log(LOGDEBUG,"Window %s was already loaded", GetProperty("xmlfile").c_str());
-    CLog::Log(LOGDEBUG,"Alloc resources: %.2fm", 1000.f * (end - start) / freq);
-  }
+  CLog::Log(LOGDEBUG,"Alloc resources: %.2fms (%.2f ms skin load)", 1000.f * (end - start) / freq, 1000.f * (slend - start) / freq);
 #endif
   m_bAllocated = true;
 }
@@ -763,13 +685,7 @@ void CGUIWindow::FreeResources(bool forceUnload /*= FALSE */)
   CGUIControlGroup::FreeResources();
   //g_TextureManager.Dump();
   // unload the skin
-  if (m_loadType == LOAD_EVERY_TIME || forceUnload) ClearAll();
-  if (forceUnload)
-  {
-    delete m_windowXMLRootElement;
-    m_windowXMLRootElement = NULL;
-    m_xmlIncludeConditions.clear();
-  }
+  if (m_loadOnDemand || forceUnload) ClearAll();
 }
 
 void CGUIWindow::DynamicResourceAlloc(bool bOnOff)
@@ -784,25 +700,13 @@ void CGUIWindow::ClearAll()
   CGUIControlGroup::ClearAll();
   m_windowLoaded = false;
   m_dynamicResourceAlloc = true;
-  m_visibleCondition.reset();
 }
 
 bool CGUIWindow::Initialize()
 {
   if (!g_windowManager.Initialized())
     return false;     // can't load if we have no skin yet
-  if(!NeedXMLReload())
-    return true;
-  if(g_application.IsCurrentThread())
-    AllocResources();
-  else
-  {
-    // if not app thread, send gui msg via app messenger
-    // and wait for results, so windowLoaded flag would be updated
-    CGUIMessage msg(GUI_MSG_WINDOW_LOAD, 0, 0);
-    CApplicationMessenger::Get().SendGUIMessage(msg, GetID(), true);
-  }
-  return m_windowLoaded;
+  return Load(GetProperty("xmlfile").asString());
 }
 
 void CGUIWindow::SetInitialVisibility()
@@ -822,7 +726,7 @@ bool CGUIWindow::CheckAnimation(ANIMATION_TYPE animType)
   // special cases first
   if (animType == ANIM_TYPE_WINDOW_CLOSE)
   {
-    if (!m_bAllocated || !HasProcessed()) // can't process an animation if we aren't allocated or haven't processed
+    if (!m_bAllocated || !m_hasRendered) // can't render an animation if we aren't allocated or haven't rendered
       return false;
     // make sure we update our visibility prior to queuing the window close anim
     for (unsigned int i = 0; i < m_children.size(); i++)
@@ -863,7 +767,7 @@ bool CGUIWindow::ControlGroupHasFocus(int groupID, int controlID)
   // 1.  Run through and get control with groupID (assume unique)
   // 2.  Get it's selected item.
   CGUIControl *group = GetFirstFocusableControl(groupID);
-  if (!group) group = GetControl(groupID);
+  if (!group) group = (CGUIControl *)GetControl(groupID);
 
   if (group && group->IsGroup())
   {
@@ -929,11 +833,7 @@ bool CGUIWindow::OnMove(int fromControl, int moveAction)
   while (control)
   { // grab the next control direction
     moveHistory.push_back(nextControl);
-    CGUIAction action = control->GetNavigateAction(moveAction);
-    action.ExecuteActions(nextControl, GetParentID());
-    nextControl = action.GetNavigation();
-    if (!nextControl) // 0 isn't valid control id
-      return false;
+    nextControl = control->GetNextControl(moveAction);
     // check our history - if the nextControl is in it, we can't focus it
     for (unsigned int i = 0; i < moveHistory.size(); i++)
     {
@@ -959,6 +859,7 @@ void CGUIWindow::SetDefaults()
   m_defaultControl = 0;
   m_posX = m_posY = m_width = m_height = 0;
   m_overlayState = OVERLAY_STATE_PARENT_WINDOW;   // Use parent or previous window's state
+  m_visibleCondition = 0;
   m_previousWindow = WINDOW_INVALID;
   m_animations.clear();
   m_origins.clear();
@@ -993,11 +894,31 @@ bool CGUIWindow::SendMessage(int message, int id, int param1 /* = 0*/, int param
   return OnMessage(msg);
 }
 
+#ifdef _DEBUG
 void CGUIWindow::DumpTextureUse()
 {
-#ifdef _DEBUG
   CLog::Log(LOGDEBUG, "%s for window %u", __FUNCTION__, GetID());
   CGUIControlGroup::DumpTextureUse();
+}
+#endif
+
+void CGUIWindow::ChangeButtonToEdit(int id, bool singleLabel /* = false*/)
+{
+#ifdef PRE_SKIN_VERSION_9_10_COMPATIBILITY
+  CGUIControl *name = (CGUIControl *)GetControl(id);
+  if (name && name->GetControlType() == CGUIControl::GUICONTROL_BUTTON)
+  { // change it to an edit control
+    CGUIEditControl *edit = new CGUIEditControl(*(const CGUIButtonControl *)name);
+    if (edit)
+    {
+      if (singleLabel)
+        edit->SetLabel("");
+      InsertControl(edit, name);
+      RemoveControl(name);
+      name->FreeResources();
+      delete name;
+    }
+  }
 #endif
 }
 
@@ -1030,12 +951,12 @@ void CGUIWindow::SetRunActionsManually()
 
 void CGUIWindow::RunLoadActions()
 {
-  m_loadActions.ExecuteActions(GetID(), GetParentID());
+  m_loadActions.Execute(GetID(), GetParentID());
 }
 
 void CGUIWindow::RunUnloadActions()
 {
-  m_unloadActions.ExecuteActions(GetID(), GetParentID());
+  m_unloadActions.Execute(GetID(), GetParentID());
 }
 
 void CGUIWindow::ClearBackground()
@@ -1045,20 +966,3 @@ void CGUIWindow::ClearBackground()
   if (color)
     g_graphicsContext.Clear(color);
 }
-
-void CGUIWindow::SetID(int id)
-{
-  CGUIControlGroup::SetID(id);
-  m_idRange.clear();
-  m_idRange.push_back(id);
-}
-
-bool CGUIWindow::HasID(int controlID) const
-{
-  for (std::vector<int>::const_iterator it = m_idRange.begin(); it != m_idRange.end() ; ++it)
-  {
-    if (controlID == *it)
-      return true;
-  }
-  return false;
-}
