@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2008 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,18 +13,18 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
+ *  along with XBMC; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  http://www.gnu.org/copyleft/gpl.html
  *
  */
 
 #include "system.h"
 #include "interfaces/Builtins.h"
 #include "ButtonTranslator.h"
-#include "profiles/ProfilesManager.h"
 #include "utils/URIUtils.h"
+#include "settings/Settings.h"
 #include "guilib/Key.h"
-#include "guilib/WindowIDs.h"
 #include "input/XBMC_keysym.h"
 #include "input/XBMC_keytable.h"
 #include "filesystem/File.h"
@@ -32,17 +32,9 @@
 #include "FileItem.h"
 #include "utils/StringUtils.h"
 #include "utils/log.h"
-#include "utils/XBMCTinyXML.h"
+#include "tinyXML/tinyxml.h"
 #include "XBIRRemote.h"
 
-#if defined(TARGET_WINDOWS)
-#include "input/windows/WINJoystick.h"
-#elif defined(HAS_SDL_JOYSTICK) || defined(HAS_EVENT_SERVER)
-#include "SDLJoystick.h"
-#endif
-
-#define JOYSTICK_DEFAULT_MAP "_xbmc_"
-
 using namespace std;
 using namespace XFILE;
 
@@ -52,12 +44,6 @@ typedef struct
   int action;
 } ActionMapping;
 
-typedef struct
-{
-  int origin;
-  int target;
-} WindowMapping;
-
 static const ActionMapping actions[] =
 {
         {"left"              , ACTION_MOVE_LEFT },
@@ -83,8 +69,6 @@ static const ActionMapping actions[] =
         {"stepback"          , ACTION_STEP_BACK},
         {"bigstepforward"    , ACTION_BIG_STEP_FORWARD},
         {"bigstepback"       , ACTION_BIG_STEP_BACK},
-        {"chapterorbigstepforward", ACTION_CHAPTER_OR_BIG_STEP_FORWARD},
-        {"chapterorbigstepback"   , ACTION_CHAPTER_OR_BIG_STEP_BACK},
         {"osd"               , ACTION_SHOW_OSD},
         {"showsubtitles"     , ACTION_SHOW_SUBTITLES},
         {"nextsubtitle"      , ACTION_NEXT_SUBTITLE},
@@ -108,8 +92,7 @@ static const ActionMapping actions[] =
         {"nextcalibration"   , ACTION_CALIBRATE_SWAP_ARROWS},
         {"resetcalibration"  , ACTION_CALIBRATE_RESET},
         {"analogmove"        , ACTION_ANALOG_MOVE},
-        {"rotate"            , ACTION_ROTATE_PICTURE_CW},
-        {"rotateccw"         , ACTION_ROTATE_PICTURE_CCW},
+        {"rotate"            , ACTION_ROTATE_PICTURE},
         {"close"             , ACTION_NAV_BACK}, // backwards compatibility
         {"subtitledelayminus", ACTION_SUBTITLE_DELAY_MIN},
         {"subtitledelay"     , ACTION_SUBTITLE_DELAY},
@@ -147,7 +130,6 @@ static const ActionMapping actions[] =
         {"rewind"            , ACTION_PLAYER_REWIND},
         {"play"              , ACTION_PLAYER_PLAY},
         {"playpause"         , ACTION_PLAYER_PLAYPAUSE},
-        {"switchplayer"      , ACTION_SWITCH_PLAYER},
         {"delete"            , ACTION_DELETE_ITEM},
         {"copy"              , ACTION_COPY_ITEM},
         {"move"              , ACTION_MOVE_ITEM},
@@ -201,7 +183,6 @@ static const ActionMapping actions[] =
         {"jumpsms7"          , ACTION_JUMP_SMS7},
         {"jumpsms8"          , ACTION_JUMP_SMS8},
         {"jumpsms9"          , ACTION_JUMP_SMS9},
-        {"filter"            , ACTION_FILTER},
         {"filterclear"       , ACTION_FILTER_CLEAR},
         {"filtersms2"        , ACTION_FILTER_SMS2},
         {"filtersms3"        , ACTION_FILTER_SMS3},
@@ -220,29 +201,6 @@ static const ActionMapping actions[] =
         {"blue"              , ACTION_TELETEXT_BLUE},
         {"increasepar"       , ACTION_INCREASE_PAR},
         {"decreasepar"       , ACTION_DECREASE_PAR},
-        {"volampup"          , ACTION_VOLAMP_UP},
-        {"volampdown"        , ACTION_VOLAMP_DOWN},
-        {"createbookmark"        , ACTION_CREATE_BOOKMARK},
-        {"createepisodebookmark" , ACTION_CREATE_EPISODE_BOOKMARK},
-        {"settingsreset"      , ACTION_SETTINGS_RESET},
-        {"settingslevelchange", ACTION_SETTINGS_LEVEL_CHANGE},
-
-        // 3D movie playback/GUI
-        {"stereomode"                , ACTION_STEREOMODE_SELECT}, // cycle 3D modes, for now an alias for next
-        {"nextstereomode"            , ACTION_STEREOMODE_NEXT},
-        {"previousstereomode"        , ACTION_STEREOMODE_PREVIOUS},
-        {"togglestereomode"          , ACTION_STEREOMODE_TOGGLE},
-        {"stereomodetomono"          , ACTION_STEREOMODE_TOMONO},
-
-        // PVR actions
-        {"channelup"             , ACTION_CHANNEL_UP},
-        {"channeldown"           , ACTION_CHANNEL_DOWN},
-        {"previouschannelgroup"  , ACTION_PREVIOUS_CHANNELGROUP},
-        {"nextchannelgroup"      , ACTION_NEXT_CHANNELGROUP},
-        {"playpvr"               , ACTION_PVR_PLAY},
-        {"playpvrtv"             , ACTION_PVR_PLAY_TV},
-        {"playpvrradio"          , ACTION_PVR_PLAY_RADIO},
-        {"record"                , ACTION_RECORD},
 
         // Mouse actions
         {"leftclick"         , ACTION_MOUSE_LEFT_CLICK},
@@ -254,17 +212,6 @@ static const ActionMapping actions[] =
         {"mousedrag"         , ACTION_MOUSE_DRAG},
         {"mousemove"         , ACTION_MOUSE_MOVE},
 
-        // Touch
-        {"tap"               , ACTION_TOUCH_TAP},
-        {"longpress"         , ACTION_TOUCH_LONGPRESS},
-        {"pangesture"        , ACTION_GESTURE_PAN},
-        {"zoomgesture"       , ACTION_GESTURE_ZOOM},
-        {"rotategesture"     , ACTION_GESTURE_ROTATE},
-        {"swipeleft"         , ACTION_GESTURE_SWIPE_LEFT},
-        {"swiperight"        , ACTION_GESTURE_SWIPE_RIGHT},
-        {"swipeup"           , ACTION_GESTURE_SWIPE_UP},
-        {"swipedown"         , ACTION_GESTURE_SWIPE_DOWN},
-
         // Do nothing action
         { "noop"             , ACTION_NOOP}
 };
@@ -279,21 +226,6 @@ static const ActionMapping windows[] =
         {"music"                    , WINDOW_MUSIC},
         {"video"                    , WINDOW_VIDEOS},
         {"videos"                   , WINDOW_VIDEO_NAV},
-        {"tv"                       , WINDOW_PVR}, // backward compat
-        {"pvr"                      , WINDOW_PVR},
-        {"pvrguideinfo"             , WINDOW_DIALOG_PVR_GUIDE_INFO},
-        {"pvrrecordinginfo"         , WINDOW_DIALOG_PVR_RECORDING_INFO},
-        {"pvrtimersetting"          , WINDOW_DIALOG_PVR_TIMER_SETTING},
-        {"pvrgroupmanager"          , WINDOW_DIALOG_PVR_GROUP_MANAGER},
-        {"pvrchannelmanager"        , WINDOW_DIALOG_PVR_CHANNEL_MANAGER},
-        {"pvrguidesearch"           , WINDOW_DIALOG_PVR_GUIDE_SEARCH},
-        {"pvrchannelscan"           , WINDOW_DIALOG_PVR_CHANNEL_SCAN},
-        {"pvrupdateprogress"        , WINDOW_DIALOG_PVR_UPDATE_PROGRESS},
-        {"pvrosdchannels"           , WINDOW_DIALOG_PVR_OSD_CHANNELS},
-        {"pvrosdguide"              , WINDOW_DIALOG_PVR_OSD_GUIDE},
-        {"pvrosddirector"           , WINDOW_DIALOG_PVR_OSD_DIRECTOR},
-        {"pvrosdcutter"             , WINDOW_DIALOG_PVR_OSD_CUTTER},
-        {"pvrosdteletext"           , WINDOW_DIALOG_OSD_TELETEXT},
         {"systeminfo"               , WINDOW_SYSTEM_INFORMATION},
         {"testpattern"              , WINDOW_TEST_PATTERN},
         {"screencalibration"        , WINDOW_SCREEN_CALIBRATION},
@@ -304,18 +236,14 @@ static const ActionMapping windows[] =
         {"musicsettings"            , WINDOW_SETTINGS_MYMUSIC},
         {"systemsettings"           , WINDOW_SETTINGS_SYSTEM},
         {"videossettings"           , WINDOW_SETTINGS_MYVIDEOS},
-        {"networksettings"          , WINDOW_SETTINGS_SERVICE}, // backward compat
-        {"servicesettings"          , WINDOW_SETTINGS_SERVICE},
+        {"networksettings"          , WINDOW_SETTINGS_NETWORK},
         {"appearancesettings"       , WINDOW_SETTINGS_APPEARANCE},
-        {"pvrsettings"              , WINDOW_SETTINGS_MYPVR},
-        {"tvsettings"               , WINDOW_SETTINGS_MYPVR},  // backward compat
         {"scripts"                  , WINDOW_PROGRAMS}, // backward compat
         {"videofiles"               , WINDOW_VIDEO_FILES},
         {"videolibrary"             , WINDOW_VIDEO_NAV},
         {"videoplaylist"            , WINDOW_VIDEO_PLAYLIST},
         {"loginscreen"              , WINDOW_LOGIN_SCREEN},
         {"profiles"                 , WINDOW_SETTINGS_PROFILES},
-        {"skinsettings"             , WINDOW_SKIN_SETTINGS},
         {"addonbrowser"             , WINDOW_ADDON_BROWSER},
         {"yesnodialog"              , WINDOW_DIALOG_YES_NO},
         {"progressdialog"           , WINDOW_DIALOG_PROGRESS},
@@ -328,6 +256,7 @@ static const ActionMapping windows[] =
         {"numericinput"             , WINDOW_DIALOG_NUMERIC},
         {"gamepadinput"             , WINDOW_DIALOG_GAMEPAD},
         {"shutdownmenu"             , WINDOW_DIALOG_BUTTON_MENU},
+        {"musicscan"                , WINDOW_DIALOG_MUSIC_SCAN},
         {"mutebug"                  , WINDOW_DIALOG_MUTE_BUG},
         {"playercontrols"           , WINDOW_DIALOG_PLAYER_CONTROLS},
         {"seekbar"                  , WINDOW_DIALOG_SEEK_BAR},
@@ -344,6 +273,8 @@ static const ActionMapping windows[] =
         {"profilesettings"          , WINDOW_DIALOG_PROFILE_SETTINGS},
         {"locksettings"             , WINDOW_DIALOG_LOCK_SETTINGS},
         {"contentsettings"          , WINDOW_DIALOG_CONTENT_SETTINGS},
+        {"videoscan"                , WINDOW_DIALOG_VIDEO_SCAN},
+        {"favourites"               , WINDOW_DIALOG_FAVOURITES},
         {"songinformation"          , WINDOW_DIALOG_SONG_INFO},
         {"smartplaylisteditor"      , WINDOW_DIALOG_SMART_PLAYLIST_EDITOR},
         {"smartplaylistrule"        , WINDOW_DIALOG_SMART_PLAYLIST_RULE},
@@ -355,7 +286,6 @@ static const ActionMapping windows[] =
         {"karaokelargeselector"     , WINDOW_DIALOG_KARAOKE_SELECTOR},
         {"sliderdialog"             , WINDOW_DIALOG_SLIDER},
         {"addoninformation"         , WINDOW_DIALOG_ADDON_INFO},
-        {"subtitlesearch"           , WINDOW_DIALOG_SUBTITLES},
         {"musicplaylist"            , WINDOW_MUSIC_PLAYLIST},
         {"musicfiles"               , WINDOW_MUSIC_FILES},
         {"musiclibrary"             , WINDOW_MUSIC_NAV},
@@ -367,7 +297,6 @@ static const ActionMapping windows[] =
         {"movieinformation"         , WINDOW_DIALOG_VIDEO_INFO},
         {"textviewer"               , WINDOW_DIALOG_TEXT_VIEWER},
         {"fullscreenvideo"          , WINDOW_FULLSCREEN_VIDEO},
-        {"fullscreenlivetv"         , WINDOW_FULLSCREEN_LIVETV}, // virtual window/keymap section for PVR specific bindings in fullscreen playback (which internally uses WINDOW_FULLSCREEN_VIDEO)
         {"visualisation"            , WINDOW_VISUALISATION},
         {"slideshow"                , WINDOW_SLIDESHOW},
         {"filestackingdialog"       , WINDOW_DIALOG_FILESTACKING},
@@ -382,10 +311,7 @@ static const ActionMapping windows[] =
         {"startwindow"              , WINDOW_START},
         {"startup"                  , WINDOW_STARTUP_ANIM},
         {"peripherals"              , WINDOW_DIALOG_PERIPHERAL_MANAGER},
-        {"peripheralsettings"       , WINDOW_DIALOG_PERIPHERAL_SETTINGS},
-        {"extendedprogressdialog"   , WINDOW_DIALOG_EXT_PROGRESS},
-        {"mediafilter"              , WINDOW_DIALOG_MEDIA_FILTER},
-        {"addon"                    , WINDOW_ADDON_START}};
+        {"peripheralsettings"       , WINDOW_DIALOG_PERIPHERAL_SETTINGS}};
 
 static const ActionMapping mousecommands[] =
 {
@@ -399,26 +325,7 @@ static const ActionMapping mousecommands[] =
   { "mousemove",   ACTION_MOUSE_MOVE }
 };
 
-static const ActionMapping touchcommands[] =
-{
-  { "tap",                ACTION_TOUCH_TAP },
-  { "longpress",          ACTION_TOUCH_LONGPRESS },
-  { "pan",                ACTION_GESTURE_PAN },
-  { "zoom",               ACTION_GESTURE_ZOOM },
-  { "rotate",             ACTION_GESTURE_ROTATE },
-  { "swipeleft",          ACTION_GESTURE_SWIPE_LEFT },
-  { "swiperight",         ACTION_GESTURE_SWIPE_RIGHT },
-  { "swipeup",            ACTION_GESTURE_SWIPE_UP },
-  { "swipedown",          ACTION_GESTURE_SWIPE_DOWN }
-};
-
-static const WindowMapping fallbackWindows[] =
-{
-  { WINDOW_FULLSCREEN_LIVETV,          WINDOW_FULLSCREEN_VIDEO },
-  { WINDOW_DIALOG_FULLSCREEN_INFO,     WINDOW_FULLSCREEN_VIDEO }
-};
-
-#ifdef TARGET_WINDOWS
+#ifdef WIN32
 static const ActionMapping appcommands[] =
 {
   { "browser_back",        APPCOMMAND_BROWSER_BACKWARD },
@@ -460,9 +367,9 @@ CButtonTranslator::CButtonTranslator()
   m_Loaded = false;
 }
 
-#if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
-void CButtonTranslator::ClearLircButtonMapEntries()
+CButtonTranslator::~CButtonTranslator()
 {
+#if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
   vector<lircButtonMap*> maps;
   for (map<CStdString,lircButtonMap*>::iterator it  = lircRemotesMap.begin();
                                                 it != lircRemotesMap.end();++it)
@@ -471,13 +378,6 @@ void CButtonTranslator::ClearLircButtonMapEntries()
   vector<lircButtonMap*>::iterator itend = unique(maps.begin(),maps.end());
   for (vector<lircButtonMap*>::iterator it = maps.begin(); it != itend;++it)
     delete *it;
-}
-#endif
-
-CButtonTranslator::~CButtonTranslator()
-{
-#if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
-  ClearLircButtonMapEntries();
 #endif
 }
 
@@ -535,7 +435,7 @@ bool CButtonTranslator::Load(bool AlwaysLoad)
       CFileItemList files;
       XFILE::CDirectory::GetDirectory(DIRS_TO_CHECK[dirIndex], files, ".xml");
       // Sort the list for filesystem based priorities, e.g. 01-keymap.xml, 02-keymap-overrides.xml
-      files.Sort(SortByFile, SortOrderAscending);
+      files.Sort(SORT_METHOD_FILE, SORT_ORDER_ASC);
       for(int fileIndex = 0; fileIndex<files.Size(); ++fileIndex)
       {
         if (!files[fileIndex]->m_bIsFolder)
@@ -554,7 +454,7 @@ bool CButtonTranslator::Load(bool AlwaysLoad)
           CFileItemList files;
           XFILE::CDirectory::GetDirectory(devicedir, files, ".xml");
           // Sort the list for filesystem based priorities, e.g. 01-keymap.xml, 02-keymap-overrides.xml
-          files.Sort(SortByFile, SortOrderAscending);
+          files.Sort(SORT_METHOD_FILE, SORT_ORDER_ASC);
           for(int fileIndex = 0; fileIndex<files.Size(); ++fileIndex)
           {
             if (!files[fileIndex]->m_bIsFolder)
@@ -572,19 +472,20 @@ bool CButtonTranslator::Load(bool AlwaysLoad)
   }
 
 #if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
-#ifdef TARGET_POSIX
+#ifdef _LINUX
 #define REMOTEMAP "Lircmap.xml"
 #else
 #define REMOTEMAP "IRSSmap.xml"
 #endif
-  CStdString lircmapPath = URIUtils::AddFileToFolder("special://xbmc/system/", REMOTEMAP);
+  CStdString lircmapPath;
+  URIUtils::AddFileToFolder("special://xbmc/system/", REMOTEMAP, lircmapPath);
   lircRemotesMap.clear();
   if(CFile::Exists(lircmapPath))
     success |= LoadLircMap(lircmapPath);
   else
     CLog::Log(LOGDEBUG, "CButtonTranslator::Load - no system %s found, skipping", REMOTEMAP);
 
-  lircmapPath = CProfilesManager::Get().GetUserDataItem(REMOTEMAP);
+  lircmapPath = g_settings.GetUserDataItem(REMOTEMAP);
   if(CFile::Exists(lircmapPath))
     success |= LoadLircMap(lircmapPath);
   else
@@ -602,7 +503,7 @@ bool CButtonTranslator::Load(bool AlwaysLoad)
 
 bool CButtonTranslator::LoadKeymap(const CStdString &keymapPath)
 {
-  CXBMCTinyXML xmlDoc;
+  TiXmlDocument xmlDoc;
 
   CLog::Log(LOGINFO, "Loading %s", keymapPath.c_str());
   if (!xmlDoc.LoadFile(keymapPath))
@@ -611,11 +512,6 @@ bool CButtonTranslator::LoadKeymap(const CStdString &keymapPath)
     return false;
   }
   TiXmlElement* pRoot = xmlDoc.RootElement();
-  if (!pRoot)
-  {
-    CLog::Log(LOGERROR, "Error getting keymap root: %s", keymapPath.c_str());
-    return false;
-  }
   CStdString strValue = pRoot->Value();
   if ( strValue != "keymap")
   {
@@ -626,7 +522,7 @@ bool CButtonTranslator::LoadKeymap(const CStdString &keymapPath)
   TiXmlNode* pWindow = pRoot->FirstChild();
   while (pWindow)
   {
-    if (pWindow->Type() == TiXmlNode::TINYXML_ELEMENT)
+    if (pWindow->Type() == TiXmlNode::ELEMENT)
     {
       int windowID = WINDOW_INVALID;
       const char *szWindow = pWindow->Value();
@@ -648,13 +544,13 @@ bool CButtonTranslator::LoadKeymap(const CStdString &keymapPath)
 #if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
 bool CButtonTranslator::LoadLircMap(const CStdString &lircmapPath)
 {
-#ifdef TARGET_POSIX
+#ifdef _LINUX
 #define REMOTEMAPTAG "lircmap"
 #else
 #define REMOTEMAPTAG "irssmap"
 #endif
   // load our xml file, and fill up our mapping tables
-  CXBMCTinyXML xmlDoc;
+  TiXmlDocument xmlDoc;
 
   // Load the config file
   CLog::Log(LOGINFO, "Loading %s", lircmapPath.c_str());
@@ -676,7 +572,7 @@ bool CButtonTranslator::LoadLircMap(const CStdString &lircmapPath)
   TiXmlNode* pRemote = pRoot->FirstChild();
   while (pRemote)
   {
-    if (pRemote->Type() == TiXmlNode::TINYXML_ELEMENT)
+    if (pRemote->Type() == TiXmlNode::ELEMENT)
     {
       const char *szRemote = pRemote->Value();
       if (szRemote)
@@ -745,7 +641,7 @@ int CButtonTranslator::TranslateLircRemoteString(const char* szDevice, const cha
 #if defined(HAS_SDL_JOYSTICK) || defined(HAS_EVENT_SERVER)
 void CButtonTranslator::MapJoystickActions(int windowID, TiXmlNode *pJoystick)
 {
-  string joyname = JOYSTICK_DEFAULT_MAP; // default global map name
+  string joyname = "_xbmc_"; // default global map name
   vector<string> joynames;
   map<int, string> buttonMap;
   map<int, string> axisMap;
@@ -847,153 +743,99 @@ void CButtonTranslator::MapJoystickActions(int windowID, TiXmlNode *pJoystick)
 
 bool CButtonTranslator::TranslateJoystickString(int window, const char* szDevice, int id, short inputType, int& action, CStdString& strAction, bool &fullrange)
 {
-  fullrange = false;
+  bool found = false;
 
-  // resolve the correct JoystickMap
+  map<string, JoystickMap>::iterator it;
   map<string, JoystickMap> *jmap;
+
+  fullrange = false;
   if (inputType == JACTIVE_AXIS)
     jmap = &m_joystickAxisMap;
   else if (inputType == JACTIVE_BUTTON)
     jmap = &m_joystickButtonMap;
   else if (inputType == JACTIVE_HAT)
-    jmap = &m_joystickHatMap;
+  	jmap = &m_joystickHatMap;
   else
   {
-    CLog::Log(LOGERROR, "Error reading joystick input type '%i'", (int) inputType);
+    CLog::Log(LOGERROR, "Error reading joystick input type");
     return false;
   }
 
-  map<string, JoystickMap>::iterator it = jmap->find(szDevice);
+  it = jmap->find(szDevice);
   if (it==jmap->end())
-  {
-    it = jmap->find(JOYSTICK_DEFAULT_MAP); // default global map name
-    if (it==jmap->end())
-      return false;
-  }
+    return false;
 
   JoystickMap wmap = it->second;
+  JoystickMap::iterator it2;
+  map<int, string> windowbmap;
+  map<int, string> globalbmap;
+  map<int, string>::iterator it3;
 
-  // try to get the action from the current window
-  action = GetActionCode(window, id, wmap, strAction, fullrange);
+  it2 = wmap.find(window);
 
-  // if it's invalid, try to get it from a fallback window or the global map
-  if (action == 0)
+  // first try local window map
+  if (it2!=wmap.end())
   {
-    int fallbackWindow = GetFallbackWindow(window);
-    if (fallbackWindow > -1)
-      action = GetActionCode(fallbackWindow, id, wmap, strAction, fullrange);
-    // still no valid action? use global map
-    if (action == 0)
-      action = GetActionCode(-1, id, wmap, strAction, fullrange);
-  }
-
-  return (action > 0);
-}
-
-bool CButtonTranslator::TranslateTouchAction(int window, int touchAction, int touchPointers, int &action)
-{
-  action = 0;
-  if (touchPointers <= 0)
-    touchPointers = 1;
-
-  touchAction += touchPointers - 1;
-  touchAction |= KEY_TOUCH;
-
-  action = GetTouchActionCode(window, touchAction);
-  if (action <= 0)
-    action = GetTouchActionCode(-1, touchAction);
-
-  return action > 0;
-}
-
-int CButtonTranslator::GetActionCode(int window, int action)
-{
-  map<int, buttonMap>::const_iterator it = m_translatorMap.find(window);
-  if (it == m_translatorMap.end())
-    return 0;
-
-  buttonMap::const_iterator it2 = it->second.find(action);
-  if (it2 == it->second.end())
-    return 0;
-
-  return it2->second.id;
-}
-
-/*
- * Translates a joystick input to an action code
- */
-int CButtonTranslator::GetActionCode(int window, int id, const JoystickMap &wmap, CStdString &strAction, bool &fullrange) const
-{
-  int action = 0;
-  bool found = false;
-
-  JoystickMap::const_iterator it = wmap.find(window);
-  if (it != wmap.end())
-  {
-    const map<int, string> &windowbmap = it->second;
-    map<int, string>::const_iterator it2 = windowbmap.find(id);
-    if (it2 != windowbmap.end())
+    windowbmap = it2->second;
+    it3 = windowbmap.find(id);
+    if (it3 != windowbmap.end())
     {
-      strAction = (it2->second).c_str();
+      strAction = (it3->second).c_str();
       found = true;
     }
-
-    it2 = windowbmap.find(abs(id)|0xFFFF0000);
-    if (it2 != windowbmap.end())
+    it3 = windowbmap.find(abs(id)|0xFFFF0000);
+    if (it3 != windowbmap.end())
     {
-      strAction = (it2->second).c_str();
+      strAction = (it3->second).c_str();
       found = true;
       fullrange = true;
     }
-
     // Hats joystick
-    it2 = windowbmap.find(id|0xFFF00000);
-    if (it2 != windowbmap.end())
+    it3 = windowbmap.find(id|0xFFF00000);
+    if (it3 != windowbmap.end())
     {
-      strAction = (it2->second).c_str();
+      strAction = (it3->second).c_str();
       found = true;
     }
   }
 
+  // if not found, try global map
+  if (!found)
+  {
+    it2 = wmap.find(-1);
+    if (it2 != wmap.end())
+    {
+      globalbmap = it2->second;
+      it3 = globalbmap.find(id);
+      if (it3 != globalbmap.end())
+      {
+        strAction = (it3->second).c_str();
+        found = true;
+      }
+      it3 = globalbmap.find(abs(id)|0xFFFF0000);
+      if (it3 != globalbmap.end())
+      {
+        strAction = (it3->second).c_str();
+        found = true;
+        fullrange = true;
+      }
+      it3 = globalbmap.find(id|0xFFF00000);
+      if (it3 != globalbmap.end())
+      {
+        strAction = (it3->second).c_str();
+        found = true;
+      }
+    }
+  }
+
+  // translated found action
   if (found)
-    TranslateActionString(strAction.c_str(), action);
-  return action;
+    return TranslateActionString(strAction.c_str(), action);
+
+  return false;
 }
 #endif
 
-void CButtonTranslator::GetActions(std::vector<std::string> &actionList)
-{
-  unsigned int size = sizeof(actions) / sizeof(ActionMapping);
-  actionList.clear();
-  actionList.reserve(size);
-  for (unsigned int index = 0; index < size; index++)
-    actionList.push_back(actions[index].name);
-}
-
-void CButtonTranslator::GetWindows(std::vector<std::string> &windowList)
-{
-  unsigned int size = sizeof(windows) / sizeof(ActionMapping);
-  windowList.clear();
-  windowList.reserve(size);
-  for (unsigned int index = 0; index < size; index++)
-    windowList.push_back(windows[index].name);
-}
-
-int CButtonTranslator::GetFallbackWindow(int windowID)
-{
-  for (unsigned int index = 0; index < sizeof(fallbackWindows) / sizeof(fallbackWindows[0]); ++index)
-  {
-    if (fallbackWindows[index].origin == windowID)
-      return fallbackWindows[index].target;
-  }
-  // for addon windows use WINDOW_ADDON_START
-  // because id is dynamic
-  if (windowID >= WINDOW_ADDON_START && windowID <= WINDOW_ADDON_END)
-    return WINDOW_ADDON_START;
-
-  return -1;
-}
-
 CAction CButtonTranslator::GetAction(int window, const CKey &key, bool fallback)
 {
   CStdString strAction;
@@ -1001,14 +843,7 @@ CAction CButtonTranslator::GetAction(int window, const CKey &key, bool fallback)
   int actionID = GetActionCode(window, key, strAction);
   // if it's invalid, try to get it from the global map
   if (actionID == 0 && fallback)
-  {
-    int fallbackWindow = GetFallbackWindow(window);
-    if (fallbackWindow > -1)
-      actionID = GetActionCode(fallbackWindow, key, strAction);
-    // still no valid action? use global map
-    if (actionID == 0)
-      actionID = GetActionCode( -1, key, strAction);
-  }
+    actionID = GetActionCode( -1, key, strAction);
   // Now fill our action structure
   CAction action(actionID, strAction, key);
   return action;
@@ -1029,7 +864,7 @@ int CButtonTranslator::GetActionCode(int window, const CKey &key, CStdString &st
     strAction = (*it2).second.strID;
     it2 = (*it).second.end();
   }
-#ifdef TARGET_POSIX
+#ifdef _LINUX
   // Some buttoncodes changed in Hardy
   if (action == 0 && (code & KEY_VKEY) == KEY_VKEY && (code & 0x0F00))
   {
@@ -1052,7 +887,6 @@ void CButtonTranslator::MapAction(uint32_t buttonCode, const char *szAction, but
   int action = ACTION_NONE;
   if (!TranslateActionString(szAction, action) || !buttonCode)
     return;   // no valid action, or an invalid buttoncode
-
   // have a valid action, and a valid button - map it.
   // check to see if we've already got this (button,action) pair defined
   buttonMap::iterator it = map.find(buttonCode);
@@ -1121,7 +955,7 @@ void CButtonTranslator::MapWindowActions(TiXmlNode *pWindow, int windowID)
       }
 
       // add our map to our table
-      if (!map.empty())
+      if (map.size() > 0)
         m_translatorMap.insert(pair<int, buttonMap>( windowID, map));
     }
   }
@@ -1137,23 +971,13 @@ void CButtonTranslator::MapWindowActions(TiXmlNode *pWindow, int windowID)
     }
   }
 #endif
-
-  if ((pDevice = pWindow->FirstChild("touch")) != NULL)
-  {
-    // map touch actions
-    while (pDevice)
-    {
-      MapTouchActions(windowID, pDevice);
-      pDevice = pDevice->NextSibling("touch");
-    }
-  }
 }
 
 bool CButtonTranslator::TranslateActionString(const char *szAction, int &action)
 {
   action = ACTION_NONE;
   CStdString strAction = szAction;
-  StringUtils::ToLower(strAction);
+  strAction.ToLower();
   if (CBuiltins::HasCommand(strAction)) 
     action = ACTION_BUILT_IN_FUNCTION;
 
@@ -1188,18 +1012,18 @@ CStdString CButtonTranslator::TranslateWindow(int windowID)
 int CButtonTranslator::TranslateWindow(const CStdString &window)
 {
   CStdString strWindow(window);
-  if (strWindow.empty()) 
+  if (strWindow.IsEmpty()) 
     return WINDOW_INVALID;
-  StringUtils::ToLower(strWindow);
+  strWindow.ToLower();
   // eliminate .xml
-  if (StringUtils::EndsWith(strWindow, ".xml"))
-    strWindow = strWindow.substr(0, strWindow.size() - 4);
+  if (strWindow.Mid(strWindow.GetLength() - 4) == ".xml" )
+    strWindow = strWindow.Mid(0, strWindow.GetLength() - 4);
 
   // window12345, for custom window to be keymapped
-  if (strWindow.length() > 6 && StringUtils::StartsWithNoCase(strWindow, "window"))
-    strWindow = strWindow.substr(6);
-  if (StringUtils::StartsWithNoCase(strWindow, "my"))  // drop "my" prefix
-    strWindow = strWindow.substr(2);
+  if (strWindow.length() > 6 && strWindow.Left(6).Equals("window"))
+    strWindow = strWindow.Mid(6);
+  if (strWindow.Left(2) == "my")  // drop "my" prefix
+    strWindow = strWindow.Mid(2);
   if (StringUtils::IsNaturalNumber(strWindow))
   {
     // allow a full window id or a delta id
@@ -1226,7 +1050,7 @@ uint32_t CButtonTranslator::TranslateGamepadString(const char *szButton)
     return 0;
   uint32_t buttonCode = 0;
   CStdString strButton = szButton;
-  StringUtils::ToLower(strButton);
+  strButton.ToLower();
   if (strButton.Equals("a")) buttonCode = KEY_BUTTON_A;
   else if (strButton.Equals("b")) buttonCode = KEY_BUTTON_B;
   else if (strButton.Equals("x")) buttonCode = KEY_BUTTON_X;
@@ -1265,7 +1089,7 @@ uint32_t CButtonTranslator::TranslateRemoteString(const char *szButton)
     return 0;
   uint32_t buttonCode = 0;
   CStdString strButton = szButton;
-  StringUtils::ToLower(strButton);
+  strButton.ToLower();
   if (strButton.Equals("left")) buttonCode = XINPUT_IR_REMOTE_LEFT;
   else if (strButton.Equals("right")) buttonCode = XINPUT_IR_REMOTE_RIGHT;
   else if (strButton.Equals("up")) buttonCode = XINPUT_IR_REMOTE_UP;
@@ -1309,17 +1133,13 @@ uint32_t CButtonTranslator::TranslateRemoteString(const char *szButton)
   else if (strButton.Equals("pageminus")) buttonCode = XINPUT_IR_REMOTE_CHANNEL_MINUS;
   else if (strButton.Equals("mute")) buttonCode = XINPUT_IR_REMOTE_MUTE;
   else if (strButton.Equals("recordedtv")) buttonCode = XINPUT_IR_REMOTE_RECORDED_TV;
-  else if (strButton.Equals("guide")) buttonCode = XINPUT_IR_REMOTE_GUIDE;
+  else if (strButton.Equals("guide")) buttonCode = XINPUT_IR_REMOTE_TITLE;   // same as title
   else if (strButton.Equals("livetv")) buttonCode = XINPUT_IR_REMOTE_LIVE_TV;
-  else if (strButton.Equals("liveradio")) buttonCode = XINPUT_IR_REMOTE_LIVE_RADIO;
-  else if (strButton.Equals("epgsearch")) buttonCode = XINPUT_IR_REMOTE_EPG_SEARCH;
   else if (strButton.Equals("star")) buttonCode = XINPUT_IR_REMOTE_STAR;
   else if (strButton.Equals("hash")) buttonCode = XINPUT_IR_REMOTE_HASH;
   else if (strButton.Equals("clear")) buttonCode = XINPUT_IR_REMOTE_CLEAR;
   else if (strButton.Equals("enter")) buttonCode = XINPUT_IR_REMOTE_ENTER;
   else if (strButton.Equals("xbox")) buttonCode = XINPUT_IR_REMOTE_DISPLAY; // same as display
-  else if (strButton.Equals("playlist")) buttonCode = XINPUT_IR_REMOTE_PLAYLIST;
-  else if (strButton.Equals("guide")) buttonCode = XINPUT_IR_REMOTE_GUIDE;
   else if (strButton.Equals("teletext")) buttonCode = XINPUT_IR_REMOTE_TELETEXT;
   else if (strButton.Equals("red")) buttonCode = XINPUT_IR_REMOTE_RED;
   else if (strButton.Equals("green")) buttonCode = XINPUT_IR_REMOTE_GREEN;
@@ -1375,17 +1195,9 @@ uint32_t CButtonTranslator::TranslateKeyboardButton(TiXmlElement *pButton)
   CStdString strKey = szButton;
   if (strKey.Equals("key"))
   {
-    std::string strID;
-    if (pButton->QueryValueAttribute("id", &strID) == TIXML_SUCCESS)
-    {
-      const char *str = strID.c_str();
-      char *endptr;
-      long int id = strtol(str, &endptr, 0);
-      if (endptr - str != (int)strlen(str) || id <= 0 || id > 0x00FFFFFF)
-        CLog::Log(LOGDEBUG, "%s - invalid key id %s", __FUNCTION__, strID.c_str());
-      else
-        button_id = (uint32_t) id;
-    }
+    int id = 0;
+    if (pButton->QueryIntAttribute("id", &id) == TIXML_SUCCESS)
+      button_id = (uint32_t)id;
     else
       CLog::Log(LOGERROR, "Keyboard Translator: `key' button has no id");
   }
@@ -1396,13 +1208,14 @@ uint32_t CButtonTranslator::TranslateKeyboardButton(TiXmlElement *pButton)
   CStdString strMod;
   if (pButton->QueryValueAttribute("mod", &strMod) == TIXML_SUCCESS)
   {
-    StringUtils::ToLower(strMod);
+    strMod.ToLower();
 
-    vector<string> modArray = StringUtils::Split(strMod, ",");
-    for (vector<string>::const_iterator i = modArray.begin(); i != modArray.end(); ++i)
+    CStdStringArray modArray;
+    StringUtils::SplitString(strMod, ",", modArray);
+    for (unsigned int i = 0; i < modArray.size(); i++)
     {
-      string substr = *i;
-      StringUtils::Trim(substr);
+      CStdString& substr = modArray[i];
+      substr.Trim();
 
       if (substr == "ctrl" || substr == "control")
         button_id |= CKey::MODIFIER_CTRL;
@@ -1412,8 +1225,6 @@ uint32_t CButtonTranslator::TranslateKeyboardButton(TiXmlElement *pButton)
         button_id |= CKey::MODIFIER_ALT;
       else if (substr == "super" || substr == "win")
         button_id |= CKey::MODIFIER_SUPER;
-      else if (substr == "meta" || substr == "cmd")
-        button_id |= CKey::MODIFIER_META;
       else
         CLog::Log(LOGERROR, "Keyboard Translator: Unknown key modifier %s in %s", substr.c_str(), strMod.c_str());
      }
@@ -1424,9 +1235,9 @@ uint32_t CButtonTranslator::TranslateKeyboardButton(TiXmlElement *pButton)
 
 uint32_t CButtonTranslator::TranslateAppCommand(const char *szButton)
 {
-#ifdef TARGET_WINDOWS
+#ifdef WIN32
   CStdString strAppCommand = szButton;
-  StringUtils::ToLower(strAppCommand);
+  strAppCommand.ToLower();
 
   for (int i = 0; i < sizeof(appcommands)/sizeof(appcommands[0]); i++)
     if (strAppCommand.Equals(appcommands[i].name))
@@ -1441,7 +1252,7 @@ uint32_t CButtonTranslator::TranslateAppCommand(const char *szButton)
 uint32_t CButtonTranslator::TranslateMouseCommand(const char *szButton)
 {
   CStdString strMouseCommand = szButton;
-  StringUtils::ToLower(strMouseCommand);
+  strMouseCommand.ToLower();
 
   for (unsigned int i = 0; i < sizeof(mousecommands)/sizeof(mousecommands[0]); i++)
     if (strMouseCommand.Equals(mousecommands[i].name))
@@ -1456,7 +1267,6 @@ void CButtonTranslator::Clear()
 {
   m_translatorMap.clear();
 #if defined(HAS_LIRC) || defined(HAS_IRSERVERSUITE)
-  ClearLircButtonMapEntries();
   lircRemotesMap.clear();
 #endif
 
@@ -1468,109 +1278,3 @@ void CButtonTranslator::Clear()
 
   m_Loaded = false;
 }
-
-uint32_t CButtonTranslator::TranslateTouchCommand(TiXmlElement *pButton, CButtonAction &action)
-{
-  const char *szButton = pButton->Value();
-  if (szButton == NULL || pButton->FirstChild() == NULL)
-    return ACTION_NONE;
-
-  const char *szAction = pButton->FirstChild()->Value();
-  if (szAction == NULL)
-    return ACTION_NONE;
-
-  CStdString strTouchCommand = szButton;
-  StringUtils::ToLower(strTouchCommand);
-
-  const char *attrVal = pButton->Attribute("direction");
-  if (attrVal != NULL)
-    strTouchCommand += attrVal;
-
-  uint32_t actionId = ACTION_NONE;
-  for (unsigned int i = 0; i < sizeof(touchcommands)/sizeof(touchcommands[0]); i++)
-  {
-    if (strTouchCommand.Equals(touchcommands[i].name))
-    {
-      actionId = touchcommands[i].action;
-      break;
-    }
-  }
-
-  if (actionId <= ACTION_NONE)
-  {
-    CLog::Log(LOGERROR, "%s: Can't find touch command %s", __FUNCTION__, szButton);
-    return ACTION_NONE;
-  }
-
-  attrVal = pButton->Attribute("pointers");
-  if (attrVal != NULL)
-  {
-    int pointers = (int)strtol(attrVal, NULL, 0);
-    if (pointers >= 1)
-      actionId += pointers - 1;
-  }
-
-  action.strID = szAction;
-  if (!TranslateActionString(szAction, action.id) || action.id <= ACTION_NONE)
-    return ACTION_NONE;
-
-  return actionId | KEY_TOUCH;
-}
-
-void CButtonTranslator::MapTouchActions(int windowID, TiXmlNode *pTouch)
-{
-  if (pTouch == NULL)
-    return;
-
-  buttonMap map;
-  // check if there already is a touch map for the window ID
-  std::map<int, buttonMap>::iterator it = m_touchMap.find(windowID);
-  if (it != m_touchMap.end())
-  {
-    // get the existing touch map and remove it from the window mapping
-    // as it will be inserted later on
-    map = it->second;
-    m_touchMap.erase(it);
-  }
-
-  uint32_t actionId = 0;
-  TiXmlElement *pTouchElem = pTouch->ToElement();
-  if (pTouchElem == NULL)
-    return;
-
-  TiXmlElement *pButton = pTouchElem->FirstChildElement();
-  while (pButton != NULL)
-  {
-    CButtonAction action;
-    actionId = TranslateTouchCommand(pButton, action);
-    if (actionId > 0)
-    {
-      // check if there already is a mapping for the parsed action
-      // and remove it if necessary
-      buttonMap::iterator actionIt = map.find(actionId);
-      if (actionIt != map.end())
-        map.erase(actionIt);
-
-      map.insert(std::make_pair(actionId, action));
-    }
-
-    pButton = pButton->NextSiblingElement();
-  }
-
-  // add the modified touch map with the window ID
-  if (!map.empty())
-    m_touchMap.insert(std::pair<int, buttonMap>(windowID, map));
-}
-
-int CButtonTranslator::GetTouchActionCode(int window, int action)
-{
-  std::map<int, buttonMap>::const_iterator windowIt = m_touchMap.find(window);
-  if (windowIt == m_touchMap.end())
-    return ACTION_NONE;
-
-  buttonMap::const_iterator touchIt = windowIt->second.find(action);
-  if (touchIt == windowIt->second.end())
-    return ACTION_NONE;
-
-  return touchIt->second.id;
-}
