@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
+ *      Copyright (C) 2005-2011 Team XBMC
  *      http://xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
@@ -13,8 +13,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
+ *  along with XBMC; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  http://www.gnu.org/copyleft/gpl.html
  *
  */
 
@@ -26,28 +27,22 @@
 #include "devices/PeripheralNIC.h"
 #include "devices/PeripheralNyxboard.h"
 #include "devices/PeripheralTuner.h"
+#if defined(HAVE_LIBCEC)
 #include "devices/PeripheralCecAdapter.h"
-#include "devices/PeripheralImon.h"
+#endif
 #include "bus/PeripheralBusUSB.h"
 #include "dialogs/GUIDialogPeripheralManager.h"
 
-#if defined(HAVE_LIBCEC)
-#include "bus/virtual/PeripheralBusCEC.h"
-#endif
-
 #include "threads/SingleLock.h"
 #include "utils/log.h"
 #include "utils/XMLUtils.h"
-#include "utils/XBMCTinyXML.h"
+#include "settings/GUISettings.h"
+#include "tinyXML/tinyxml.h"
 #include "filesystem/Directory.h"
 #include "guilib/GUIWindowManager.h"
 #include "guilib/LocalizeStrings.h"
 #include "dialogs/GUIDialogKaiToast.h"
-#include "GUIUserMessages.h"
 #include "utils/StringUtils.h"
-#include "Util.h"
-#include "guilib/Key.h"
-#include "settings/lib/Setting.h"
 
 using namespace PERIPHERALS;
 using namespace XFILE;
@@ -55,6 +50,8 @@ using namespace std;
 
 CPeripherals::CPeripherals(void)
 {
+  CDirectory::Create("special://profile/peripheral_data");
+
   Clear();
 }
 
@@ -76,17 +73,12 @@ void CPeripherals::Initialise(void)
   {
     m_bIsStarted = true;
 
-    CDirectory::Create("special://profile/peripheral_data");
-
     /* load mappings from peripherals.xml */
     LoadMappings();
 
 #if defined(HAVE_PERIPHERAL_BUS_USB)
     m_busses.push_back(new CPeripheralBusUSB(this));
 #endif
-#if defined(HAVE_LIBCEC)
-    m_busses.push_back(new CPeripheralBusCEC(this));
-#endif
 
     /* initialise all known busses */
     for (int iBusPtr = (int)m_busses.size() - 1; iBusPtr >= 0; iBusPtr--)
@@ -114,9 +106,9 @@ void CPeripherals::Clear(void)
   /* delete mappings */
   for (unsigned int iMappingPtr = 0; iMappingPtr < m_mappings.size(); iMappingPtr++)
   {
-    map<CStdString, PeripheralDeviceSetting> settings = m_mappings.at(iMappingPtr).m_settings;
-    for (map<CStdString, PeripheralDeviceSetting>::iterator itr = settings.begin(); itr != settings.end(); ++itr)
-      delete itr->second.m_setting;
+    map<CStdString, CSetting *> settings = m_mappings.at(iMappingPtr).m_settings;
+    for (map<CStdString, CSetting *>::iterator itr = settings.begin(); itr != settings.end(); itr++)
+      delete itr->second;
     m_mappings.at(iMappingPtr).m_settings.clear();
   }
   m_mappings.clear();
@@ -145,8 +137,9 @@ void CPeripherals::TriggerDeviceScan(const PeripheralBusType type /* = PERIPHERA
 
 CPeripheralBus *CPeripherals::GetBusByType(const PeripheralBusType type) const
 {
-  CSingleLock lock(m_critSection);
   CPeripheralBus *bus(NULL);
+
+  CSingleLock lock(m_critSection);
   for (unsigned int iBusPtr = 0; iBusPtr < m_busses.size(); iBusPtr++)
   {
     if (m_busses.at(iBusPtr)->Type() == type)
@@ -161,8 +154,8 @@ CPeripheralBus *CPeripherals::GetBusByType(const PeripheralBusType type) const
 
 CPeripheral *CPeripherals::GetPeripheralAtLocation(const CStdString &strLocation, PeripheralBusType busType /* = PERIPHERAL_BUS_UNKNOWN */) const
 {
-  CSingleLock lock(m_critSection);
   CPeripheral *peripheral(NULL);
+  CSingleLock lock(m_critSection);
   for (unsigned int iBusPtr = 0; iBusPtr < m_busses.size(); iBusPtr++)
   {
     /* check whether the bus matches if a bus type other than unknown was passed */
@@ -197,8 +190,8 @@ CPeripheralBus *CPeripherals::GetBusWithDevice(const CStdString &strLocation) co
 
 int CPeripherals::GetPeripheralsWithFeature(vector<CPeripheral *> &results, const PeripheralFeature feature, PeripheralBusType busType /* = PERIPHERAL_BUS_UNKNOWN */) const
 {
-  CSingleLock lock(m_critSection);
   int iReturn(0);
+  CSingleLock lock(m_critSection);
   for (unsigned int iBusPtr = 0; iBusPtr < m_busses.size(); iBusPtr++)
   {
     /* check whether the bus matches if a bus type other than unknown was passed */
@@ -229,50 +222,54 @@ bool CPeripherals::HasPeripheralWithFeature(const PeripheralFeature feature, Per
   return (GetPeripheralsWithFeature(dummy, feature, busType) > 0);
 }
 
-CPeripheral *CPeripherals::CreatePeripheral(CPeripheralBus &bus, const PeripheralScanResult& result)
+CPeripheral *CPeripherals::CreatePeripheral(CPeripheralBus &bus, const PeripheralType type, const CStdString &strLocation, int iVendorId /* = 0 */, int iProductId /* = 0 */)
 {
   CPeripheral *peripheral = NULL;
-  PeripheralScanResult mappedResult = result;
-  if (mappedResult.m_busType == PERIPHERAL_BUS_UNKNOWN)
-    mappedResult.m_busType = bus.Type();
-
   /* check whether there's something mapped in peripherals.xml */
-  if (!GetMappingForDevice(bus, mappedResult))
+  PeripheralType mappedType = type;
+  CStdString strDeviceName;
+  int iMappingPtr = GetMappingForDevice(bus, type, iVendorId, iProductId);
+  bool bHasMapping(iMappingPtr >= 0);
+  if (bHasMapping)
+  {
+    mappedType    = m_mappings[iMappingPtr].m_mappedTo;
+    strDeviceName = m_mappings[iMappingPtr].m_strDeviceName;
+  }
+  else
   {
     /* don't create instances for devices that aren't mapped in peripherals.xml */
     return NULL;
   }
 
-  switch(mappedResult.m_mappedType)
+  switch(mappedType)
   {
   case PERIPHERAL_HID:
-    peripheral = new CPeripheralHID(mappedResult);
+    peripheral = new CPeripheralHID(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_NIC:
-    peripheral = new CPeripheralNIC(mappedResult);
+    peripheral = new CPeripheralNIC(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_DISK:
-    peripheral = new CPeripheralDisk(mappedResult);
+    peripheral = new CPeripheralDisk(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_NYXBOARD:
-    peripheral = new CPeripheralNyxboard(mappedResult);
+    peripheral = new CPeripheralNyxboard(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_TUNER:
-    peripheral = new CPeripheralTuner(mappedResult);
+    peripheral = new CPeripheralTuner(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_BLUETOOTH:
-    peripheral = new CPeripheralBluetooth(mappedResult);
+    peripheral = new CPeripheralBluetooth(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
     break;
 
   case PERIPHERAL_CEC:
 #if defined(HAVE_LIBCEC)
-    if (bus.Type() == PERIPHERAL_BUS_CEC)
-      peripheral = new CPeripheralCecAdapter(mappedResult);
+    peripheral = new CPeripheralCecAdapter(type, bus.Type(), strLocation, strDeviceName, iVendorId, iProductId);
 #else
     if (!m_bMissingLibCecWarningDisplayed)
     {
@@ -283,10 +280,6 @@ CPeripheral *CPeripherals::CreatePeripheral(CPeripheralBus &bus, const Periphera
 #endif
     break;
 
-  case PERIPHERAL_IMON:
-    peripheral = new CPeripheralImon(mappedResult);
-    break;
-
   default:
     break;
   }
@@ -297,11 +290,13 @@ CPeripheral *CPeripherals::CreatePeripheral(CPeripheralBus &bus, const Periphera
      * Initialise() will make sure that each device is only initialised once */
     if (peripheral->Initialise())
     {
+      if (!bHasMapping)
+        peripheral->SetHidden(true);
       bus.Register(peripheral);
     }
     else
     {
-      CLog::Log(LOGDEBUG, "%s - failed to initialise peripheral on '%s'", __FUNCTION__, mappedResult.m_strLocation.c_str());
+      CLog::Log(LOGDEBUG, "%s - failed to initialise peripheral on '%s'", __FUNCTION__, strLocation.c_str());
       delete peripheral;
       peripheral = NULL;
     }
@@ -316,15 +311,7 @@ void CPeripherals::OnDeviceAdded(const CPeripheralBus &bus, const CPeripheral &p
   if (dialog && dialog->IsActive())
     dialog->Update();
 
-  // refresh settings (peripherals manager could be enabled now)
-  CGUIMessage msg(GUI_MSG_UPDATE, WINDOW_SETTINGS_SYSTEM, 0);
-  g_windowManager.SendThreadMessage(msg, WINDOW_SETTINGS_SYSTEM);
-
-  SetChanged();
-
-  // don't show a notification for devices detected during the initial scan
-  if (bus.IsInitialised())
-    CGUIDialogKaiToast::QueueNotification(CGUIDialogKaiToast::Info, g_localizeStrings.Get(35005), peripheral.DeviceName());
+  CGUIDialogKaiToast::QueueNotification(CGUIDialogKaiToast::Info, g_localizeStrings.Get(35005), peripheral.DeviceName());
 }
 
 void CPeripherals::OnDeviceDeleted(const CPeripheralBus &bus, const CPeripheral &peripheral)
@@ -333,16 +320,10 @@ void CPeripherals::OnDeviceDeleted(const CPeripheralBus &bus, const CPeripheral
   if (dialog && dialog->IsActive())
     dialog->Update();
 
-  // refresh settings (peripherals manager could be disabled now)
-  CGUIMessage msg(GUI_MSG_UPDATE, WINDOW_SETTINGS_SYSTEM, 0);
-  g_windowManager.SendThreadMessage(msg, WINDOW_SETTINGS_SYSTEM);
-
-  SetChanged();
-
   CGUIDialogKaiToast::QueueNotification(CGUIDialogKaiToast::Info, g_localizeStrings.Get(35006), peripheral.DeviceName());
 }
 
-bool CPeripherals::GetMappingForDevice(const CPeripheralBus &bus, PeripheralScanResult& result) const
+int CPeripherals::GetMappingForDevice(const CPeripheralBus &bus, const PeripheralType classType, int iVendorId, int iProductId) const
 {
   /* check all mappings in the order in which they are defined in peripherals.xml */
   for (unsigned int iMappingPtr = 0; iMappingPtr < m_mappings.size(); iMappingPtr++)
@@ -357,26 +338,24 @@ bool CPeripherals::GetMappingForDevice(const CPeripheralBus &bus, PeripheralScan
     else
     {
       for (unsigned int i = 0; i < mapping.m_PeripheralID.size(); i++)
-        if (mapping.m_PeripheralID[i].m_iVendorId == result.m_iVendorId && mapping.m_PeripheralID[i].m_iProductId == result.m_iProductId)
+        if (mapping.m_PeripheralID[i].m_iVendorId == iVendorId && mapping.m_PeripheralID[i].m_iProductId == iProductId)
           bProductMatch = true;
     }
 
     bool bBusMatch = (mapping.m_busType == PERIPHERAL_BUS_UNKNOWN || mapping.m_busType == bus.Type());
-    bool bClassMatch = (mapping.m_class == PERIPHERAL_UNKNOWN || mapping.m_class == result.m_type);
+    bool bClassMatch = (mapping.m_class == PERIPHERAL_UNKNOWN || mapping.m_class == classType);
 
     if (bProductMatch && bBusMatch && bClassMatch)
     {
       CStdString strVendorId, strProductId;
-      PeripheralTypeTranslator::FormatHexString(result.m_iVendorId, strVendorId);
-      PeripheralTypeTranslator::FormatHexString(result.m_iProductId, strProductId);
+      PeripheralTypeTranslator::FormatHexString(iVendorId, strVendorId);
+      PeripheralTypeTranslator::FormatHexString(iProductId, strProductId);
       CLog::Log(LOGDEBUG, "%s - device (%s:%s) mapped to %s (type = %s)", __FUNCTION__, strVendorId.c_str(), strProductId.c_str(), mapping.m_strDeviceName.c_str(), PeripheralTypeTranslator::TypeToString(mapping.m_mappedTo));
-      result.m_mappedType    = m_mappings[iMappingPtr].m_mappedTo;
-      result.m_strDeviceName = m_mappings[iMappingPtr].m_strDeviceName;
-      return true;
+      return iMappingPtr;
     }
   }
 
-  return false;
+  return -1;
 }
 
 void CPeripherals::GetSettingsFromMapping(CPeripheral &peripheral) const
@@ -403,15 +382,15 @@ void CPeripherals::GetSettingsFromMapping(CPeripheral &peripheral) const
 
     if (bBusMatch && bProductMatch && bClassMatch)
     {
-      for (map<CStdString, PeripheralDeviceSetting>::const_iterator itr = mapping->m_settings.begin(); itr != mapping->m_settings.end(); ++itr)
-        peripheral.AddSetting((*itr).first, (*itr).second.m_setting, (*itr).second.m_order);
+      for (map<CStdString, CSetting *>::const_iterator itr = mapping->m_settings.begin(); itr != mapping->m_settings.end(); itr++)
+        peripheral.AddSetting((*itr).first, (*itr).second);
     }
   }
 }
 
 bool CPeripherals::LoadMappings(void)
 {
-  CXBMCTinyXML xmlDoc;
+  TiXmlDocument xmlDoc;
   if (!xmlDoc.LoadFile("special://xbmc/system/peripherals.xml"))
   {
     CLog::Log(LOGWARNING, "%s - peripherals.xml does not exist", __FUNCTION__);
@@ -427,6 +406,7 @@ bool CPeripherals::LoadMappings(void)
 
   for (TiXmlElement *currentNode = pRootElement->FirstChildElement("peripheral"); currentNode; currentNode = currentNode->NextSiblingElement("peripheral"))
   {
+    CStdStringArray vpArray, idArray;
     PeripheralID id;
     PeripheralDeviceMapping mapping;
 
@@ -436,18 +416,18 @@ bool CPeripherals::LoadMappings(void)
     if (currentNode->Attribute("vendor_product"))
     {
       // The vendor_product attribute is a list of comma separated vendor:product pairs
-      vector<string> vpArray = StringUtils::Split(currentNode->Attribute("vendor_product"), ",");
-      for (vector<string>::const_iterator i = vpArray.begin(); i != vpArray.end(); ++i)
+      StringUtils::SplitString(currentNode->Attribute("vendor_product"), ",", vpArray);
+      for (unsigned int i = 0; i < vpArray.size(); i++)
       {
-        vector<string> idArray = StringUtils::Split(*i, ":");
+        StringUtils::SplitString(vpArray[i], ":", idArray);
         if (idArray.size() != 2)
         {
           CLog::Log(LOGERROR, "%s - ignoring node \"%s\" with invalid vendor_product attribute", __FUNCTION__, mapping.m_strDeviceName.c_str());
           continue;
         }
 
-        id.m_iVendorId = PeripheralTypeTranslator::HexStringToInt(idArray[0].c_str());
-        id.m_iProductId = PeripheralTypeTranslator::HexStringToInt(idArray[1].c_str());
+        id.m_iVendorId = PeripheralTypeTranslator::HexStringToInt(idArray[0]);
+        id.m_iProductId = PeripheralTypeTranslator::HexStringToInt(idArray[1]);
         mapping.m_PeripheralID.push_back(id);
       }
     }
@@ -464,16 +444,14 @@ bool CPeripherals::LoadMappings(void)
   return true;
 }
 
-void CPeripherals::GetSettingsFromMappingsFile(TiXmlElement *xmlNode, map<CStdString, PeripheralDeviceSetting> &settings)
+void CPeripherals::GetSettingsFromMappingsFile(TiXmlElement *xmlNode, map<CStdString, CSetting *> &m_settings)
 {
   TiXmlElement *currentNode = xmlNode->FirstChildElement("setting");
-  int iMaxOrder = 0;
-
   while (currentNode)
   {
     CSetting *setting = NULL;
     CStdString strKey(currentNode->Attribute("key"));
-    if (strKey.empty())
+    if (strKey.IsEmpty())
       continue;
 
     CStdString strSettingsType(currentNode->Attribute("type"));
@@ -488,7 +466,7 @@ void CPeripherals::GetSettingsFromMappingsFile(TiXmlElement *xmlNode, map<CStdSt
       bool bValue = (strcmp(currentNode->Attribute("value"), "no") != 0 &&
                      strcmp(currentNode->Attribute("value"), "false") != 0 &&
                      strcmp(currentNode->Attribute("value"), "0") != 0);
-      setting = new CSettingBool(strKey, iLabelId, bValue);
+      setting = new CSettingBool(0, strKey, iLabelId, bValue, CHECKMARK_CONTROL);
     }
     else if (strSettingsType.Equals("int"))
     {
@@ -496,7 +474,8 @@ void CPeripherals::GetSettingsFromMappingsFile(TiXmlElement *xmlNode, map<CStdSt
       int iMin   = currentNode->Attribute("min") ? atoi(currentNode->Attribute("min")) : 0;
       int iStep  = currentNode->Attribute("step") ? atoi(currentNode->Attribute("step")) : 1;
       int iMax   = currentNode->Attribute("max") ? atoi(currentNode->Attribute("max")) : 255;
-      setting = new CSettingInt(strKey, iLabelId, iValue, iMin, iStep, iMax);
+      CStdString strFormat(currentNode->Attribute("format"));
+      setting = new CSettingInt(0, strKey, iLabelId, iValue, iMin, iStep, iMax, SPIN_CONTROL_INT, strFormat);
     }
     else if (strSettingsType.Equals("float"))
     {
@@ -504,67 +483,28 @@ void CPeripherals::GetSettingsFromMappingsFile(TiXmlElement *xmlNode, map<CStdSt
       float fMin   = currentNode->Attribute("min") ? (float) atof(currentNode->Attribute("min")) : 0;
       float fStep  = currentNode->Attribute("step") ? (float) atof(currentNode->Attribute("step")) : 0;
       float fMax   = currentNode->Attribute("max") ? (float) atof(currentNode->Attribute("max")) : 0;
-      setting = new CSettingNumber(strKey, iLabelId, fValue, fMin, fStep, fMax);
-    }
-    else if (strSettingsType.Equals("enum"))
-    {
-      CStdString strEnums(currentNode->Attribute("lvalues"));
-      if (!strEnums.empty())
-      {
-        vector< pair<int,int> > enums;
-        vector<std::string> valuesVec;
-        StringUtils::Tokenize(strEnums, valuesVec, "|");
-        for (unsigned int i = 0; i < valuesVec.size(); i++)
-          enums.push_back(make_pair(atoi(valuesVec[i].c_str()), atoi(valuesVec[i].c_str())));
-        int iValue = currentNode->Attribute("value") ? atoi(currentNode->Attribute("value")) : 0;
-        setting = new CSettingInt(strKey, iLabelId, iValue, enums);
-      }
+      setting = new CSettingFloat(0, strKey, iLabelId, fValue, fMin, fStep, fMax, SPIN_CONTROL_FLOAT);
     }
     else
     {
       CStdString strValue(currentNode->Attribute("value"));
-      setting = new CSettingString(strKey, iLabelId, strValue);
-    }
-
-    if (setting)
-    {
-      //TODO add more types if needed
-
-      /* set the visibility */
-      setting->SetVisible(bConfigurable);
-
-      /* set the order */
-      int iOrder = 0;
-      currentNode->Attribute("order", &iOrder);
-      /* if the order attribute is invalid or 0, then the setting will be added at the end */
-      if (iOrder < 0)
-        iOrder = 0;
-      if (iOrder > iMaxOrder)
-       iMaxOrder = iOrder;
-
-      /* and add this new setting */
-      PeripheralDeviceSetting deviceSetting = { setting, iOrder };
-      settings[strKey] = deviceSetting;
+      setting = new CSettingString(0, strKey, iLabelId, strValue, EDIT_CONTROL_INPUT, !bConfigurable, -1);
     }
 
+    //TODO add more types if needed
+    setting->SetVisible(bConfigurable);
+    m_settings[strKey] = setting;
     currentNode = currentNode->NextSiblingElement("setting");
   }
-
-  /* add the settings without an order attribute or an invalid order attribute set at the end */
-  for (map<CStdString, PeripheralDeviceSetting>::iterator it = settings.begin(); it != settings.end(); ++it)
-  {
-    if (it->second.m_order == 0)
-      it->second.m_order = ++iMaxOrder;
-  }
 }
 
 void CPeripherals::GetDirectory(const CStdString &strPath, CFileItemList &items) const
 {
-  if (!StringUtils::StartsWithNoCase(strPath, "peripherals://"))
+  if (!strPath.Left(14).Equals("peripherals://"))
     return;
 
-  CStdString strPathCut = strPath.substr(14);
-  CStdString strBus = strPathCut.substr(0, strPathCut.find('/'));
+  CStdString strPathCut = strPath.Right(strPath.length() - 14);
+  CStdString strBus = strPathCut.Left(strPathCut.Find('/'));
 
   CSingleLock lock(m_critSection);
   for (unsigned int iBusPtr = 0; iBusPtr < m_busses.size(); iBusPtr++)
@@ -576,11 +516,11 @@ void CPeripherals::GetDirectory(const CStdString &strPath, CFileItemList &items)
 
 CPeripheral *CPeripherals::GetByPath(const CStdString &strPath) const
 {
-  if (!StringUtils::StartsWithNoCase(strPath, "peripherals://"))
+  if (!strPath.Left(14).Equals("peripherals://"))
     return NULL;
 
-  CStdString strPathCut = strPath.substr(14);
-  CStdString strBus = strPathCut.substr(0, strPathCut.find('/'));
+  CStdString strPathCut = strPath.Right(strPath.length() - 14);
+  CStdString strBus = strPathCut.Left(strPathCut.Find('/'));
 
   CSingleLock lock(m_critSection);
   for (unsigned int iBusPtr = 0; iBusPtr < m_busses.size(); iBusPtr++)
@@ -591,141 +531,3 @@ CPeripheral *CPeripherals::GetByPath(const CStdString &strPath) const
 
   return NULL;
 }
-
-bool CPeripherals::OnAction(const CAction &action)
-{
-  if (action.GetID() == ACTION_MUTE)
-  {
-    return ToggleMute();
-  }
-
-  if (SupportsCEC() && action.GetAmount() && (action.GetID() == ACTION_VOLUME_UP || action.GetID() == ACTION_VOLUME_DOWN))
-  {
-    vector<CPeripheral *> peripherals;
-    if (GetPeripheralsWithFeature(peripherals, FEATURE_CEC))
-    {
-      for (unsigned int iPeripheralPtr = 0; iPeripheralPtr < peripherals.size(); iPeripheralPtr++)
-      {
-        CPeripheralCecAdapter *cecDevice = (CPeripheralCecAdapter *) peripherals.at(iPeripheralPtr);
-        if (cecDevice && cecDevice->HasAudioControl())
-        {
-          if (action.GetID() == ACTION_VOLUME_UP)
-            cecDevice->VolumeUp();
-          else
-            cecDevice->VolumeDown();
-          return true;
-        }
-      }
-    }
-  }
-
-  return false;
-}
-
-bool CPeripherals::IsMuted(void)
-{
-  vector<CPeripheral *> peripherals;
-  if (SupportsCEC() && GetPeripheralsWithFeature(peripherals, FEATURE_CEC))
-  {
-    for (unsigned int iPeripheralPtr = 0; iPeripheralPtr < peripherals.size(); iPeripheralPtr++)
-    {
-      CPeripheralCecAdapter *cecDevice = (CPeripheralCecAdapter *) peripherals.at(iPeripheralPtr);
-      if (cecDevice && cecDevice->IsMuted())
-        return true;
-    }
-  }
-
-  return false;
-}
-
-bool CPeripherals::ToggleMute(void)
-{
-  vector<CPeripheral *> peripherals;
-  if (SupportsCEC() && GetPeripheralsWithFeature(peripherals, FEATURE_CEC))
-  {
-    for (unsigned int iPeripheralPtr = 0; iPeripheralPtr < peripherals.size(); iPeripheralPtr++)
-    {
-      CPeripheralCecAdapter *cecDevice = (CPeripheralCecAdapter *) peripherals.at(iPeripheralPtr);
-      if (cecDevice && cecDevice->HasAudioControl())
-      {
-        cecDevice->ToggleMute();
-        return true;
-      }
-    }
-  }
-
-  return false;
-}
-
-bool CPeripherals::ToggleDeviceState(CecStateChange mode /*= STATE_SWITCH_TOGGLE */, unsigned int iPeripheral /*= 0 */)
-{
-  bool ret(false);
-  vector<CPeripheral *> peripherals;
-
-  if (SupportsCEC() && GetPeripheralsWithFeature(peripherals, FEATURE_CEC))
-  {
-    for (unsigned int iPeripheralPtr = iPeripheral; iPeripheralPtr < peripherals.size(); iPeripheralPtr++)
-    {
-      CPeripheralCecAdapter *cecDevice = (CPeripheralCecAdapter *) peripherals.at(iPeripheralPtr);
-      if (cecDevice)
-        ret = cecDevice->ToggleDeviceState(mode);
-      if (iPeripheral)
-        break;
-    }
-  }
-
-  return ret;
-}
-
-bool CPeripherals::GetNextKeypress(float frameTime, CKey &key)
-{
-  vector<CPeripheral *> peripherals;
-  if (SupportsCEC() && GetPeripheralsWithFeature(peripherals, FEATURE_CEC))
-  {
-    for (unsigned int iPeripheralPtr = 0; iPeripheralPtr < peripherals.size(); iPeripheralPtr++)
-    {
-      CPeripheralCecAdapter *cecDevice = (CPeripheralCecAdapter *) peripherals.at(iPeripheralPtr);
-      if (cecDevice && cecDevice->GetButton())
-      {
-        CKey newKey(cecDevice->GetButton(), cecDevice->GetHoldTime());
-        cecDevice->ResetButton();
-        key = newKey;
-        return true;
-      }
-    }
-  }
-
-  return false;
-}
-
-void CPeripherals::OnSettingChanged(const CSetting *setting)
-{
-  if (setting == NULL)
-    return;
-
-  const std::string &settingId = setting->GetId();
-  if (settingId == "locale.language")
-  {
-    // user set language, no longer use the TV's language
-    vector<CPeripheral *> cecDevices;
-    if (g_peripherals.GetPeripheralsWithFeature(cecDevices, FEATURE_CEC) > 0)
-    {
-      for (vector<CPeripheral *>::iterator it = cecDevices.begin(); it != cecDevices.end(); ++it)
-        (*it)->SetSetting("use_tv_menu_language", false);
-    }
-  }
-}
-
-void CPeripherals::OnSettingAction(const CSetting *setting)
-{
-  if (setting == NULL)
-    return;
-
-  const std::string &settingId = setting->GetId();
-  if (settingId == "input.peripherals")
-  {
-    CGUIDialogPeripheralManager *dialog = (CGUIDialogPeripheralManager *)g_windowManager.GetWindow(WINDOW_DIALOG_PERIPHERAL_MANAGER);
-    if (dialog != NULL)
-      dialog->DoModal();
-  }
-}
